from __future__ import absolute_import, division, print_function, unicode_literals
import os
import argparse
import glob


def parseArguments():
    """
        Parse command line arguments

        :returns: int, int, int, str, bool, bool -- Number of steps per epoch,
            column to plot in the X axis, column to plot in the Y axis, name of
            the files containing the simulation data, whether to plot the data
            as points, wether to plot the data as lines
    """
    desc = "Generates output for gnuplot\n"\
           "It MUST be run from the root epoch folder (i.e., where it can find the folders 0/, 1/, 2/, ... lastEpoch/"\
           "To be run for example like: \">python generateGnuplotFile.py | gnuplot -persist\""
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("steps", type=int, default=4, help="Pele steps per run")
    parser.add_argument("xcol", type=int, default=2, help="xcol")
    parser.add_argument("ycol", type=int, default=4, help="ycol")
    parser.add_argument("filename", type=str, default="report_", help="Report filename")
    parser.add_argument("-points", action="store_true", help="Plot using points")
    parser.add_argument("-lines", action="store_true", help="Plot using lines")
    parser.add_argument("-zcol", type=int, default=None, help="Column to define color according to metric")
    parser.add_argument("-t", "--traj_range", type=str, default=None, help="Range of trajs to select, e.g to select trajs from 1 to 10, 1:10")

    args = parser.parse_args()
    return args.steps, args.xcol, args.ycol, args.filename, args.points, args.lines, args.zcol, args.traj_range


def generateNestedString(gnuplotString, reportName, column1, column2, stepsPerRun, printWithLines, totalNumberOfSteps=False, replotFirst=False, paletteModifier=None, trajs_range=None):
    """
        Generate a string to be passed to gnuplot

        :param gnuplotString: Template string for gnuplot
        :type gnuplotString: str
        :param reportName: Name of the files containing the simulation data
        :type reportName: str
        :param column1: Column to plot in the X axis
        :type column1: int
        :param column2: Column to plot in the Y axis
        :type column2: int
        :param stepsPerRun: Number of steps per epoch,
        :type stepsPerRun: int
        :param TotalNumberOfSteps: Not only considering steps in current epoch,
            but steps with all previous epochs
        :type TotalNumberOfSteps: bool
        :param replotFirst: Deprecated parameter
        :type replotFirst: bool
        :param paletteModifier: Wheter to use the epoch as color or a column
        :type paletteModifier: int

        :returns: str -- String to plot using gnuplot
    """
    allFolders = os.listdir('.')
    epochFolders = [epoch for epoch in allFolders if epoch.isdigit()]
    numberOfEpochs = int(len(epochFolders))

    dictionary = {'reportName': reportName, 'col2': column2, 'numberOfEpochs': numberOfEpochs, 'withLines': ''}

    if trajs_range is not None:
        start, end = map(int, traj_range.split(":"))
        dictionary['startTraj'] = start
        dictionary['runsPerEpoch'] = end
    else:
        dictionary['startTraj'] = 1
        # runs of epoch 0, assumed constant
        numberOfRunsPerEpoch = len(glob.glob(os.path.join(str(0), reportName+"*")))
        dictionary['runsPerEpoch'] = numberOfRunsPerEpoch

    if printWithLines:
        dictionary['withLines'] = "w l"

    if isinstance(column1, int):
        if totalNumberOfSteps:
            dictionary['col1'] = "($" + str(column1) + "+ (%d*j))" % stepsPerRun  # adds steps per runs, so that it mathes the total number of steps
        else:
            dictionary['col1'] = str(column1)

    return gnuplotString % dictionary + "\n"


def generatePrintString(stepsPerRun, xcol, ycol, reportName, kindOfPrint, paletteModifier, trajs_range):
    """
        Generate a template string to use with gnuplot

        :param stepsPerRun: Number of steps per epoch,
        :type stepsPerRun: int
        :param xcol: Column to plot in the X axis
        :type xcol: int
        :param ycol: Column to plot in the Y axis
        :type ycol: int
        :param reportName: Name of the files containing the simulation data
        :type reportName: str
        :param kindOfPrint:  Kind of lines to plot (solid or points)
        :type kindOfPrint: bool
        :param paletteModifier: Third column to specify color
        :type paletteModifier: int
        :trajs_range: Range of trajectories to plot
        :type trajs_range: str

        :returns: str -- String to plot using gnuplot
    """
    if kindOfPrint == "PRINT_RMSD_STEPS":
        printWithLines = True
        totalNumberOfSteps = True
    elif kindOfPrint == "PRINT_BE_RMSD":
        printWithLines = False
        totalNumberOfSteps = False
    if paletteModifier is None:
        stringPalette = "frac j/%(numberOfEpochs)d. "
        colorMetric = ""
    else:
        stringPalette = ""
        colorMetric = ":%d" % paletteModifier

    gnuplotString = "".join(["plot for [i=%(startTraj)d:%(runsPerEpoch)d] for [j=0:%(numberOfEpochs)d-1] \'\'.j.\'/%(reportName)s\'.i u %(col1)s:%(col2)d", colorMetric, " lt 6 lc palette ", stringPalette, "notitle %(withLines)s"])
    return generateNestedString(gnuplotString, reportName, xcol, ycol, stepsPerRun, printWithLines, totalNumberOfSteps, False, paletteModifier, trajs_range)


if __name__ == "__main__":
    steps_Run, Xcol, Ycol, filename, be, rmsd, colModifier, traj_range = parseArguments()
    # VARIABLES TO SET WHEN PRINTING
    if be:
        kind_Print = "PRINT_BE_RMSD"
    elif rmsd:
        kind_Print = "PRINT_RMSD_STEPS"

    printLine = generatePrintString(steps_Run, Xcol, Ycol, filename, kind_Print, colModifier, traj_range)
    print(printLine)



from AdaptivePELE.constants import blockNames


class SIMULATION_TYPE:
    PELE, MD, TEST = list(range(3))

SIMULATION_TYPE_TO_STRING_DICTIONARY = {
    SIMULATION_TYPE.PELE: blockNames.SimulationType.pele,
    SIMULATION_TYPE.MD: blockNames.SimulationType.md,
    SIMULATION_TYPE.TEST: blockNames.SimulationType.test
}


class EXITCONDITION_TYPE:
    METRIC, METRICMULTIPLETRAJS, CLUSTERING = list(range(3))

EXITCONDITION_TYPE_TO_STRING_DICTIONARY = {
    EXITCONDITION_TYPE.METRIC: blockNames.ExitConditionType.metric,
    EXITCONDITION_TYPE.METRICMULTIPLETRAJS: blockNames.ExitConditionType.metricMultipleTrajs,
    EXITCONDITION_TYPE.CLUSTERING: blockNames.ExitConditionType.clustering
}



from AdaptivePELE.constants import blockNames


class CLUSTERING_TYPES:
    rmsd, contactMap, lastSnapshot = list(range(3))

CLUSTERING_TYPE_TO_STRING_DICTIONARY = {
    CLUSTERING_TYPES.rmsd: blockNames.ClusteringTypes.rmsd,
    CLUSTERING_TYPES.contactMap: blockNames.ClusteringTypes.contactMap,
    CLUSTERING_TYPES.lastSnapshot: blockNames.ClusteringTypes.lastSnapshot
}


class SIMILARITY_TYPES:
    differenceDistance, Jaccard, correlation = list(range(3))

SIMILARITY_TYPES_TO_STRING_DICTIONARY = {
    SIMILARITY_TYPES.differenceDistance: blockNames.ClusteringTypes.differenceDistance,
    SIMILARITY_TYPES.Jaccard: blockNames.ClusteringTypes.Jaccard,
    SIMILARITY_TYPES.correlation: blockNames.ClusteringTypes.correlation
}

SIMILARITY_TYPES_NAMES = {blockNames.ClusteringTypes.differenceDistance,
                              blockNames.ClusteringTypes.Jaccard,
                              blockNames.ClusteringTypes.correlation}



from AdaptivePELE.constants import blockNames


class DENSITY_CALCULATOR_TYPES:
    null, heaviside, continuous, exitContinous = list(range(4))

DENSITY_CALCULATOR_TYPE_TO_STRING_DICTIONARY = {
    DENSITY_CALCULATOR_TYPES.heaviside: blockNames.DensityCalculator.heaviside,
    DENSITY_CALCULATOR_TYPES.null: blockNames.DensityCalculator.null,
    DENSITY_CALCULATOR_TYPES.continuous: blockNames.DensityCalculator.continuous,
    DENSITY_CALCULATOR_TYPES.exitContinous: blockNames.DensityCalculator.exitContinuous
}



import numpy
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path
from distutils.extension import Extension
try:
    from Cython.Build import cythonize
    from Cython.Distutils import build_ext
except ImportError:
    use_cython = False
else:
    use_cython = True
from distutils.command.sdist import sdist as _sdist

# Run the following line to compile atomset package
# python setup.py build_ext --inplace


class sdist(_sdist):
    def run(self):
        # Make sure the compiled Cython files in the distribution are
        # up-to-date
        from Cython.Build import cythonize
        cythonize(['cython/mycythonmodule.pyx'])
        _sdist.run(self)
        cmdclass['sdist'] = sdist

here = path.abspath(path.dirname(__file__))
ext_modules = []
cmdclass = {}
# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

if use_cython:
    ext_modules += [
        Extension("AdaptivePELE.atomset.atomset", ["AdaptivePELE/atomset/atomset.pyx"], include_dirs=["AdaptivePELE", "AdaptivePELE/atomset"]),
        Extension("AdaptivePELE.atomset.SymmetryContactMapEvaluator", ["AdaptivePELE/atomset/SymmetryContactMapEvaluator.pyx"], include_dirs=["AdaptivePELE", "AdaptivePELE/atomset"]),
        Extension("AdaptivePELE.atomset.RMSDCalculator", ["AdaptivePELE/atomset/RMSDCalculator.pyx"], include_dirs=["AdaptivePELE", "AdaptivePELE/atomset"]),
        Extension("AdaptivePELE.freeEnergies.utils", ["AdaptivePELE/freeEnergies/utils.pyx"], include_dirs=["AdaptivePELE", "AdaptivePELE/freeEnergies"])
    ]
    cmdclass.update({'build_ext': build_ext})
else:
    ext_modules += [
        Extension("AdaptivePELE.atomset.atomset", ["AdaptivePELE/atomset/atomset.c"], include_dirs=["AdaptivePELE", "AdaptivePELE/atomset"]),
        Extension("AdaptivePELE.atomset.SymmetryContactMapEvaluator", ["AdaptivePELE/atomset/SymmetryContactMapEvaluator.c"], include_dirs=["AdaptivePELE", "AdaptivePELE/atomset"]),
        Extension("AdaptivePELE.atomset.RMSDCalculator", ["AdaptivePELE/atomset/RMSDCalculator.c"], include_dirs=["AdaptivePELE", "AdaptivePELE/atomset"]),
        Extension("AdaptivePELE.freeEnergies.utils", ["AdaptivePELE/freeEnergies/utils.c"], include_dirs=["AdaptivePELE", "AdaptivePELE/freeEnergies"])
    ]

setup(
    name="AdaptivePELE",
    version="1.4.2",
    description='Enhanced sampling of molecular simulations',
    long_description=long_description,
    url="https://github.com/cescgina/AdaptivePELE",
    author='Daniel Lecina, Joan Francesc Gilabert',
    author_email='danilecina@gmail.com, cescgina@gmail.com',
    license='',
    packages=find_packages(exclude=['docs', 'tests']),
    package_data={"AdaptivePELE/atomset": ['*.pxd']},
    install_requires=['numpy'],
    cmdclass=cmdclass,
    ext_modules=cythonize(ext_modules),  # accepts a glob pattern
    include_dirs=[numpy.get_include()]
)



import sys
import os
import argparse

AMINOACIDS = ["VAL", "ASN", "GLY", "LEU", "ILE",
              "SER", "ASP", "LYS", "MET", "GLN",
              "TRP", "ARG", "ALA", "THR", "PRO",
              "PHE", "GLU", "HIS", "HIP", "TYR",
              "CYS", "HID"]

TER_CONSTR = 5

BACK_CONSTR = 0.5

CONSTR_ATOM = '''{{ "type": "constrainAtomToPosition", "springConstant": {0}, "equilibriumDistance": 0.0, "constrainThisAtom": "{1}:{2}:{3}" }},'''

CONSTR_DIST = '''{{ "type": "constrainAtomsDistance", "springConstant": {}, "equilibriumDistance": {}, "constrainThisAtom": "{}:{}:{}", "toThisOtherAtom": "{}:{}:{}" }},'''

CONSTR_CALPHA = '''{{ "type": "constrainAtomToPosition", "springConstant": {2}, "equilibriumDistance": 0.0, "constrainThisAtom": "{0}:{1}:_CA_" }},'''

class ConstraintBuilder(object):

    def __init__(self, pdb, gaps, metals):
        self.pdb = pdb
        self.gaps = gaps
        self.metals = metals

    def parse_atoms(self, interval=10):
        residues = {}
        initial_res = None
        with open(self.pdb, "r") as pdb:
            for line in pdb:
                resname = line[16:21].strip()
                atomtype = line[11:16].strip()
                resnum = line[22:26].strip()
                chain = line[20:23].strip()
                if line.startswith("ATOM") and resname in AMINOACIDS and atomtype == "CA":
                    try:
                        if not initial_res:
                            residues["initial"] = [chain, line[22:26].strip()]
                            initial_res = True
                            continue
                        # Apply constraint every 10 residues
                        elif int(resnum) % interval != 1:
                            residues["terminal"] = [chain, line[22:26].strip()]
                        elif int(resnum) % interval == 1 and line.startswith("ATOM") and resname in AMINOACIDS and atomtype == "CA":
                            residues[resnum] = chain
                    except ValueError:
                        continue
        return residues

    def build_constraint(self, residues, BACK_CONSTR=BACK_CONSTR, TER_CONSTR=TER_CONSTR):

        init_constr = ['''"constraints":[''', ]

        back_constr = [CONSTR_CALPHA.format(chain, resnum, BACK_CONSTR) for resnum, chain in residues.items() if resnum.isdigit()]

        gaps_constr = self.gaps_constraints()

        metal_constr = self.metal_constraints()

        terminal_constr = [CONSTR_CALPHA.format(residues["initial"][0], residues["initial"][1], TER_CONSTR), CONSTR_CALPHA.format(residues["terminal"][0], residues["terminal"][1], TER_CONSTR).strip(",")]

        final_constr = ["],"]

        constraints = init_constr + back_constr + gaps_constr + metal_constr + terminal_constr + final_constr

        return constraints

    def gaps_constraints(self):
        #self.gaps = {}
        gaps_constr = []
        for chain, residues in self.gaps.items():
            gaps_constr = [CONSTR_ATOM.format(TER_CONSTR, chain, terminal, "_CA_") for terminals in residues for terminal in terminals]
        return gaps_constr

    def metal_constraints(self):

        metal_constr = []
        for metal, ligands in self.metals.items():
            metal_name, chain, metnum = metal.split(" ")
            for ligand in ligands:
                ligand_info, bond_lenght = ligand
                resname, resnum, chain, ligname = ligand_info.split(" ")
                metal_constr.append(CONSTR_DIST.format(TER_CONSTR, bond_lenght, chain, resnum, ligname, chain, metnum, metal_name))
        return metal_constr


def retrieve_constraints(pdb_file, gaps, metal, back_constr=BACK_CONSTR, ter_constr=TER_CONSTR, interval=10):
    constr = ConstraintBuilder(pdb_file, gaps, metal)
    residues = constr.parse_atoms(interval=interval)
    constraints = constr.build_constraint(residues, back_constr, ter_constr)
    return constraints

def parseargs():
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('pdb', type=str, help='pdb to create the contraints on')
    parser.add_argument('conf', help='Control file to fill in. It need to templetazide with $CONSTRAINTS')
    parser.add_argument('--interval', type=int, help="Every how many CA to constraint")
    parser.add_argument('--ca', type=float, help="Constraint value to use on backbone CA", default=BACK_CONSTR)
    parser.add_argument('--terminal', type=float, help="Constraint value to use on terminal CA", default=TER_CONSTR)
    args = parser.parse_args()
    return os.path.abspath(args.pdb), os.path.abspath(args.conf), args.interval, args.conf, args.ca, args.terminal

if __name__ == "__main__":
    pdb, conf, interval, conf, back_constr, ter_constr = parseargs()
    constraints = retrieve_constraints(pdb, {}, {}, back_constr, ter_constr, interval)



import argparse
import json
import sys
from AdaptivePELE.freeEnergies import cluster
from AdaptivePELE.freeEnergies import estimate


def parseArgs():
    parser = argparse.ArgumentParser(description="Build a MSM using PyEMMA "
                                     "package")
    parser.add_argument('controlFile', type=str)
    args = parser.parse_args()
    return args


def readParams(control_file):
    try:
        with open(control_file, "r") as f:
            params = json.load(f)
    except:
        sys.exit("Sth wrong with the control file: %s" % control_file)

    trajectoryFolder = params["trajectoryFolder"]
    trajectoryBasename = params["trajectoryBasename"]
    numClusters = params["numClusters"]
    stride = params.get("stride", 1)
    lagtimes = params.get("lagtimes", [])
    lagtime = params.get("lagtime", None)
    numPCCA = params.get("numPCCA", None)
    itsOutput = params.get("itsOutput", None)
    numberOfITS = params.get("numberOfITS", -1)
    errors = params.get("errors", False)
    mlags = params.get("mlags", None)
    clusterCountsThreshold = params.get("clusterCountsThreshold", 0)

    return trajectoryFolder, trajectoryBasename, numClusters, stride, lagtimes, numPCCA, itsOutput, numberOfITS, errors, mlags, lagtime, clusterCountsThreshold


def main(control_file):

    # parameters
    trajectoryFolder, trajectoryBasename, numClusters, stride, lagtimes, _, _, numberOfITS, _, _, lagtime, clusterCountsThreshold = readParams(control_file)

    # program
    clusteringObject = cluster.Cluster(numClusters, trajectoryFolder, trajectoryBasename, alwaysCluster=False, stride=stride)
    clusteringObject.clusterTrajectories()
    clusteringObject.eliminateLowPopulatedClusters(clusterCountsThreshold)
    calculateMSM = estimate.MSM(error=False, dtrajs=clusteringObject.dtrajs)
    calculateMSM.estimate(lagtime=lagtime, lagtimes=lagtimes, numberOfITS=numberOfITS)


if __name__ == "__main__":
    arguments = parseArgs()
    main(arguments.controlFile)



from __future__ import absolute_import, division, print_function, unicode_literals
import os
import socket
machine = socket.getfqdn()

print("MACHINE", machine)
if "bsccv" in machine:
    PELE_EXECUTABLE = "/data/EAPM/PELE/PELE++/bin/rev12360/Pele_rev12360_mpi"
    DATA_FOLDER = "/data/EAPM/PELE/PELE++/data/rev12360/Data"
    DOCUMENTS_FOLDER = "/data/EAPM/PELE/PELE++/Documents/rev12360"

    PYTHON = "/data2/apps/PYTHON/2.7.5/bin/python2.7"

elif "mn.bsc" in machine:
    PELE_EXECUTABLE = "/gpfs/projects/bsc72/PELE++/bin/rev12360/Pele_rev12360_mpi"
    DATA_FOLDER = "/gpfs/projects/bsc72/PELE++/data/rev12360/Data"
    DOCUMENTS_FOLDER = "/gpfs/projects/bsc72/PELE++/Documents/rev12360"
    PYTHON = "python"

elif "bsc.mn" in machine:
    PELE_EXECUTABLE = "/gpfs/projects/bsc72/PELE++/mniv/rev12455/bin/PELE-1.5_mpi"
    DATA_FOLDER = "/gpfs/projects/bsc72/PELE++/mniv/rev12455/Data"
    DOCUMENTS_FOLDER = "/gpfs/projects/bsc72/PELE++/mniv/rev12455/Documents"

inputFileTemplate = "{ \"files\" : [ { \"path\" : \"%s\" } ] }"
trajectoryBasename = "*traj*"


class OutputPathConstants():
    """
        Class with constants that depend on the outputPath
    """
    def __init__(self, outputPath):
        self.originalControlFile = ""
        self.epochOutputPathTempletized = ""
        self.clusteringOutputDir = ""
        self.clusteringOutputObject = ""
        self.tmpInitialStructuresTemplate = ""
        self.tmpControlFilename = ""
        self.tmpInitialStructuresEquilibrationTemplate = ""
        self.tmpControlFilenameEqulibration = ""
        self.buildConstants(outputPath)

    def buildConstants(self, outputPath):
        self.buildOutputPathConstants(outputPath)

        self.tmpFolder = "tmp_" + outputPath.replace("/", "_")

        self.buildTmpFolderConstants(self.tmpFolder)

    def buildOutputPathConstants(self, outputPath):
        self.originalControlFile = os.path.join(outputPath, "originalControlFile.conf")
        self.epochOutputPathTempletized = os.path.join(outputPath, "%d")
        self.clusteringOutputDir = os.path.join(self.epochOutputPathTempletized, "clustering")
        self.clusteringOutputObject = os.path.join(self.clusteringOutputDir, "object.pkl")

    def buildTmpFolderConstants(self, tmpFolder):
        self.tmpInitialStructuresTemplate = tmpFolder+"/initial_%d_%d.pdb"
        self.tmpInitialStructuresEquilibrationTemplate = tmpFolder+"/initial_equilibration_%d.pdb"
        self.tmpControlFilename = tmpFolder+"/controlFile%d.conf"
        self.tmpControlFilenameEqulibration = tmpFolder+"/controlFile_equilibration_%d.conf"



"""
    Write specified cluster representative structures to pdb
"""
from AdaptivePELE.utilities import clusteringUtilities
import argparse


def parseArgs():
    """
        Parse command line arguments

        :returns: object -- Object containing command line options
    """
    parser = argparse.ArgumentParser(description="Write the requested cluster "
                                     "structures from a clustering object")
    parser.add_argument('clObject', type=str, help="Path to the clustering object")
    parser.add_argument('outputPath', type=str,
                        help="Path where to write the structures, including "
                        "name of the files, i.e output/path/cluster.pdb")
    parser.add_argument("structures", nargs='*', type=int, default=None,
                        help="Structures to write")
    parser.add_argument("--threshold", type=float, default=None,
                        help="Only print those structures with matching threshold")
    args = parser.parse_args()
    return args


def main(clObject, structures, condition, outputPath):
    clusteringUtilities.writeStructures(clObject, structures, condition, outputPath)


if __name__ == "__main__":
    arguments = parseArgs()
    if arguments.threshold is not None:
        condition = lambda x: abs(x.threshold-arguments.threshold) < 0.01
    else:
        condition = None
    main(arguments.clObject, arguments.structures, condition, arguments.outputPath)



from __future__ import absolute_import, division, print_function, unicode_literals
import os
try:
    import cPickle as pickle
except ImportError:
    import pickle
from AdaptivePELE.utilities import utilities


def writeStructures(clusteringObject, listStructures, checker=lambda x: True, outputPath="cluster.pdb"):
    """
        Print all clusters in listStructures that meet the condition specified
        by the checker

        :param clusteringObject: Clustering object with clusters to print
        :type clusteringObject: :py:class:`.Clustering`
        :param checker: Lambda function with the checker that should evaluate to True for intersted structures
        :type checker: function
        :param outputPath: Output cluster pdb filename
        :type outputPath: str
    """
    clObject = utilities.readClusteringObject(clusteringObject)
    nameStructure = os.path.splitext(outputPath)
    outputName = nameStructure[0]+'_%d'+nameStructure[1]
    path = os.path.split(outputName)
    pathToWrite = path[1]
    if path[0]:
        utilities.makeFolder(path[0])
        pathToWrite = os.path.join(path[0], path[1])

    if listStructures is None or len(listStructures) == 0:  # If no listStructures, write all
        listStructures = range(len(clObject.clusters.clusters))

    for element in listStructures:
        cluster = clObject.clusters.clusters[element]
        if checker is None or checker(cluster):
            print("Writing", pathToWrite % element)
            cluster.pdb.pdb += "\nENDMDL\n"
            cluster.writePDB(pathToWrite % element)



from __future__ import absolute_import, division, print_function, unicode_literals
import numpy as np
import argparse
# import matplotlib.pyplot as plt
# from mpl_toolkits.mplot3d import Axes3D
from AdaptivePELE.utilities import utilities


def parseArgs():
    desc = "Write the conformation network to a PDB file\n"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("clusteringObject", type=str, help="Path to the clustering object")
    parser.add_argument("metricCol", type=int, help="Column of the metric of interest")
    parser.add_argument("filename", type=str, help="Name of the pdb file")
    args = parser.parse_args()
    return args.clusteringObject, args.metricCol, args.filename


def writePDB(X, Y, Z, metric, f):
    templateLine = "HETATM%s  H%sCLT L 502    %s%s%s  0.75%s           H\n"

    content = ""
    for i, xn in enumerate(X):
        number = str(i).rjust(5)
        number3 = str(i).ljust(3)
        yn = Y[i]
        zn = Z[i]
        x = ("%.3f" % xn).rjust(8)
        y = ("%.3f" % yn).rjust(8)
        z = ("%.3f" % zn).rjust(8)
        g = ("%.3f" % metric[i]).rjust(8)

        content += templateLine % (number, number3, x, y, z, g)
    f.write(content)
    return f


def writeConnectionsPDB(network, f):
    template = "CONECT%s%s%s%s%s%s%s%s%s%s              \n"
    for node in network.nbunch_iter():
        contents = [node]
        for _, target in network.out_edges(node):
            contents.append(target)
            if len(contents) == 10:
                f.write(template % tuple([str(x).rjust(5) for x in contents]))
                contents = [node]
        if contents:
            contents += [""]*(10-len(contents))
            f.write(template % tuple([str(x).rjust(5) for x in contents]))
    return f


def readClustering(clusteringPath, metricCol):
    print("Reading clustering object...")
    clustering = utilities.readClusteringObject(clusteringPath)
    network = clustering.conformationNetwork.network
    metrics = [cl.metrics[metricCol] for cl in clustering.clusterIterator()]
    return clustering, network, metrics


def getCoords(clustering):
    print("Extracting nodes coordinates...")
    coords = [cl.pdb.getCOM() for cl in clustering.clusterIterator()]
    # Attempt to delete clustering object from memory, since it is not
    # needed and is very heavy
    del clustering
    coords = np.array(coords)
    Xn = coords[:, 0]
    Yn = coords[:, 1]
    Zn = coords[:, 2]
    return Xn, Yn, Zn

if __name__ == "__main__":
    clustering_path, metric_col, filename = parseArgs()

    cluster, Network, Metric = readClustering(clustering_path, metric_col)
    Xm, Ym, Zm = getCoords(cluster)

    # Write info in pdb file
    with open(filename, "w") as fw:
        fw = writePDB(Xm, Ym, Zm, Metric, fw)
        fw = writeConnectionsPDB(Network, fw)

# Plot using matplotlib

# Xe = []
# Ye = []
# Ze = []
# for e in Network.edges_iter():
#     Xe += [[coords[e[0], 0], coords[e[1], 0]]]
#     Ye += [[coords[e[0], 1], coords[e[1], 1]]]
#     Ze += [[coords[e[0], 2], coords[e[1], 2]]]
# fig = plt.figure()
# ax = fig.add_subplot(111, projection='3d')
# # ax.plot(Xe, Ye, Xe)
# for i, xl in enumerate(Xe):
#     ax.plot(xl, Ye[i], Ze[i], 'k', alpha=0.1, linewidth=0.5)
# ax.scatter(Xn, Yn, Zn, c=metric)
# # plt.colorbar()
# plt.show()



"""
    Script that computes the absolute binding free energy of an MSM.
    It needs:
        1) Trajectory wildcard in order to compute cluster volume
        2) A discretized/clusterCenters.dat file with the cluster center
        3) A MSM_object.pkl obtained with pyemma in order to obtain the stationary distribution
        4) For the moment, it needs of a reweightingT, in order to do a histogram reweighting, but does not seem to work that well
"""
from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import range
import numpy as np
import glob
import sys
import argparse
try:
    import cPickle
except ImportError:
    import pickle as cPickle
from pyemma.coordinates.clustering import AssignCenters
from AdaptivePELE.freeEnergies import runMarkovChainModel as run
from AdaptivePELE.freeEnergies import utils
import itertools


def assignNewTrajectories(trajs, clusterCenters):
    assign = AssignCenters(clusterCenters)
    dTrajs = assign.assign(trajs)
    return dTrajs


def expandTrajs(trajList):
    d = 0.5
    combinations = np.array([[0, 1, 0], [1, 0, 0], [0, 0, 1], [0, -1, 0],
                             [-1, 0, 0], [0, 0, -1]])
    return list(trajList+combinations*d)


def parseArgs():
    parser = argparse.ArgumentParser(description="Script that computes delta G")
    parser.add_argument('trajectory', type=str)
    parser.add_argument('-r', '--reweight', type=float, default=1000)
    args = parser.parse_args()
    return args.trajectory, args.reweight


def writePDB(pmf_xyzg, title="clusters.pdb"):
    # templateLine = "HETATM%s  H%sCLT L 502    %s%s%s  0.75%s           H\n"
    templateLine = "HETATM%s H%sCLT L 502    %s%s%s  0.75%s                H\n"

    content = ""
    for i, line in enumerate(pmf_xyzg):
        number = str(i).rjust(5)
        number3 = str(i).ljust(3)
        x = ("%.3f" % line[0]).rjust(8)
        y = ("%.3f" % line[1]).rjust(8)
        z = ("%.3f" % line[2]).rjust(8)
        g = ("%.3f" % line[-1]).rjust(8)

        content += templateLine % (number, number3, x, y, z, g)
    f = open(title, 'w')
    f.write(content)
    f.close()


def calcQ(T, gpmf):
    kb = 0.0019872041
    Q = np.sum(np.exp(-gpmf/(kb*T)))
    return Q


def reweightProbabilities(T, Torig, origProb):
    """
        Torig: temperature at which simulations were run
    """
    kb = 0.0019872041
    gpmf = -kb*Torig*np.log(origProb)
    gpmf[gpmf == -np.inf] = np.inf
    gpmf -= gpmf.min()  # It does not make any difference here

    print(gpmf[gpmf == np.inf])
    print(gpmf[gpmf == -np.inf])
    a = (T-Torig) / (kb * T * Torig)
    correction1 = np.exp(a * gpmf)
    beta = 1/kb/T
    betaOrig = 1/kb/Torig
    Qorig = np.sum(np.exp(-betaOrig*gpmf))
    Qnew = np.sum(np.exp(-beta*gpmf))
    correction2 = Qorig / Qnew

    return correction1 * correction2 * origProb


def loadMSM(MSMFile):
    with open(MSMFile, "rb") as MSMfile:
        MSM_object = cPickle.load(MSMfile)
    return MSM_object


def reestimate_transition_matrix(count_matrix):
    count_matrix += 1/float(count_matrix.shape[0])
    return utils.buildRevTransitionMatrix(count_matrix)


def ensure_connectivity(MSMObject, allClusters):
    if len(allClusters) == MSMObject.stationary_distribution.size:
        pi = MSMObject.stationary_distribution
        clusters = allClusters[MSMObject.connected_sets[0]]
    else:
        ######################
        # Reconstruct stationary distribution with pseudocounts to ensure
        # connectivity
        print("Adding pseudocounts to enforce connectivity")
        counts = MSMObject.count_matrix_full
        trans = reestimate_transition_matrix(counts)
        _, eic = run.getSortedEigen(trans)
        pi = run.getStationaryDistr(eic[:, 0])
        clusters = allClusters
    return pi, clusters


def gather_coordinates(originalFilenames):
    originalCoordinates = []
    for originalFilename in originalFilenames:
        trajOriginalCoordinates = list(np.loadtxt(originalFilename, ndmin=2)[:, 1:])
        if np.random.random() < 0.0:
            # Add artificial points nearby to improve volume estimation, set
            # randomly since its very slow
            sys.stderr.write("Introducing artificial neighbours\n")
            newCoords = map(expandTrajs, trajOriginalCoordinates)
            trajOriginalCoordinates.extend(list(itertools.chain.from_iterable(newCoords)))
        originalCoordinates.append(np.array(trajOriginalCoordinates))
    return originalCoordinates


def create_box(clusters, originalCoordinates, d):
    """
        Create the box discretization for the estimation of volumes
    """
    dimensions = clusters.shape[1]
    maxval = dimensions*[-np.inf]
    minval = dimensions*[np.inf]
    for coord in originalCoordinates:
        cmaxval = coord.max(axis=0)
        cminval = coord.min(axis=0)
        maxval = np.maximum(cmaxval, maxval)
        minval = np.minimum(cminval, minval)

    print("Maximum bounds", maxval, "Minimum bounds", minval)

    # Rounded floor and ceiling in intervals of "d" (e.g., floor of 1.73 with d = 0.5, will be 1.5 instead of 1.0, in order to optimize box creation.
    # An extra box is included in the ceiling, so that all the points are contained in the range given by arange
    bins = np.array([np.arange(np.floor(minval[i]) + d*int((minval[i] - np.floor(minval[i]))/d),
                               np.ceil(maxval[i]) + d*(int((maxval[i] - np.ceil(maxval[i]))/d) + 1),
                               d) for i in range(3)])
    return bins


def calculate_microstate_volumes(clusters, originalCoordinates, bins, d):
    """
        Estimate the clusters volumes using a cubic discretization of volumes
    """
    numberOfClusters = clusters.shape[0]
    print("Number of clusters", numberOfClusters)
    histogram = np.array([])
    histograms = []
    microstateVolume = np.zeros(numberOfClusters)

    # dtrajs = clusteringObject.assign(originalCoordinates)
    dtrajs = assignNewTrajectories(originalCoordinates, clusters)
    for i in range(numberOfClusters):
        allCoords = []
        for trajOriginalCoordinates, dtraj in zip(originalCoordinates, dtrajs):
            assert dtraj.shape[0] == trajOriginalCoordinates.shape[0]
            belongingFrames = np.argwhere(dtraj == i)
            trajCoords = trajOriginalCoordinates[belongingFrames, :3]
            trajCoords = trajCoords.flatten().tolist()

            allCoords.extend(trajCoords)

        allCoords = np.reshape(allCoords, (-1, 3))

        current_hist, _ = np.histogramdd(allCoords, bins=bins)
        histograms.append(current_hist)

        if histogram.size == 0:
            histogram = np.copy(current_hist)
        else:
            histogram += current_hist

    nRows, nCols, nDepth = histogram.shape
    pseudo = False
    for i in range(numberOfClusters):
        histogramCluster = histograms[i]
        if pseudo:
            # Add "pseudocounts" to try to fill the holes that lead to volume
            # underestimation compared to Matlab script for free energies
            histogramTotal = histogram.copy()
            for x, y, z in zip(*np.where(histogramCluster)):
                upBound = max(x-1, 0)
                lowBound = min(x+2, nRows)
                leftBound = max(0, y-1)
                rightBound = min(y+2, nCols)
                topBound = max(z-1, 0)
                botBound = min(z+2, nDepth)
                signsCluster = np.sign(histogramCluster[upBound:lowBound, leftBound:rightBound, topBound:botBound])
                # signs = np.sign(histogramTotal[upBound:lowBound, leftBound:rightBound, topBound:botBound])
                histogramCluster[upBound:lowBound, leftBound:rightBound, topBound:botBound] += (1-signsCluster)*d/8  # + signsCluster*d/2
                histogramTotal[upBound:lowBound, leftBound:rightBound, topBound:botBound] += (1-signsCluster)  # + signs * d/2
            histogramTotal = histogramTotal[histogramCluster > 0]
        else:
            histogramTotal = histogram[histogramCluster > 0]
        histogramCluster = histogramCluster[histogramCluster > 0]
        microstateVolume[i] = (histogramCluster/histogramTotal).sum() * d**3
    return microstateVolume


def calculate_microstate_volumes_new(clusters, originalCoordinates, bins, d):
    """
        Estimate the clusters volumes using a cubic discretization of volumes
    """
    numberOfClusters = clusters.shape[0]
    print("Number of clusters", numberOfClusters)

    allCoords = []
    # The coordinates array is built through lists in order to be able to
    # process trajectories of different lenght
    for coord in originalCoordinates:
        allCoords.extend(coord[:, :3].tolist())
    allCoords = np.array(allCoords)

    histogram, _ = np.histogramdd(allCoords, bins=bins)

    centers_trajs = []
    for i, matrix in enumerate(histogram):
        centers_x = []
        x_val = bins[0][i]
        for j, row in enumerate(matrix):
            y_val = bins[1][j]
            indices, = np.nonzero(row)
            if indices.size > 0:
                init = indices[0]
                end = indices[-1]
                centers_x.extend([[x_val, y_val, bins[2][zi]] for zi in range(init, end+1)])
        if len(centers_x):
            centers_trajs.append(np.array(centers_x))

    dtrajs = assignNewTrajectories(centers_trajs, clusters[:, :3])
    microstateVolume = np.zeros(numberOfClusters)
    for traj in dtrajs:
        counts, _ = np.histogram(traj, bins=np.arange(-0.5, numberOfClusters))
        microstateVolume += counts
    microstateVolume *= d**3
    # provisional return for debugging and testing script
    # return microstateVolume, centers_trajs, dtrajs
    return microstateVolume


def calculate_pmf(microstateVolume, pi):
    """
        Compute a potential of mean force given a stationary distribution
        (probabilities) and cluster volumes
    """
    kb = 0.0019872041
    T = 300
    beta = 1 / (kb * T)
    newDist = pi/microstateVolume
    newDist /= newDist[newDist != np.inf].sum()
    gpmf = -kb*T*np.log(newDist)
    print(gpmf[gpmf == -np.inf])
    print(gpmf[gpmf == np.inf])
    gpmf[gpmf == -np.inf] = np.inf  # to avoid contribution later
    gpmf -= gpmf.min()

    deltaW = -gpmf[gpmf != np.inf].max()
    print("bound    Delta G     Delta W     Binding Volume:     Binding Volume contribution")

    upperGpmfValues = np.arange(0, -deltaW, 0.5)

    # Initialize string variable in case loop is not accessed
    string = ""

    for upperGpmfValue in upperGpmfValues:
        bindingVolume = 0
        for g, volume in zip(gpmf, microstateVolume):
            if g <= upperGpmfValue:
                bindingVolume += np.exp(-beta * g) * volume
        deltaG = deltaW - kb*T*np.log(bindingVolume/1661)
        string = "%.1f\t%.3f\t%.3f\t%.3f\t%.3f" % (upperGpmfValue, deltaG, deltaW, bindingVolume, -kb*T*np.log(bindingVolume/1661))
        print(string)
    return gpmf, string


def main(trajWildcard, reweightingT=1000):
    allClusters = np.loadtxt("discretized/clusterCenters.dat")
    MSMObject = loadMSM('MSM_object.pkl')

    pi, clusters = ensure_connectivity(MSMObject, allClusters)
    # radius of the cube for volume determination
    d = 0.75

    originalFilenames = glob.glob(trajWildcard)
    # originalFilenames = glob.glob("rawData/"+trajWildcard)
    originalCoordinates = gather_coordinates(originalFilenames)

    bins = create_box(clusters, originalCoordinates, d)
    method = "new"
    if method == "new":
        print("Using new volume estimation with radius %.2f" % d)
        microstateVolume = calculate_microstate_volumes_new(clusters, originalCoordinates, bins, d)
    else:
        print("Using old volume estimation with radius %.2f" % d)
        microstateVolume = calculate_microstate_volumes(clusters, originalCoordinates, bins, d)
    np.savetxt("volumeOfClusters.dat", microstateVolume)

    gpmf, string = calculate_pmf(microstateVolume, pi)

    pmf_xyzg = np.hstack((clusters, np.expand_dims(gpmf, axis=1)))
    np.savetxt("pmf_xyzg.dat", pmf_xyzg)

    writePDB(pmf_xyzg)
    return string

if __name__ == "__main__":
    trajWildcard_name, reweight = parseArgs()
    main(trajWildcard_name, reweight)



from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import range
import networkx as nx
from AdaptivePELE.utilities import utilities
from AdaptivePELE.atomset import RMSDCalculator


def weight(pathway, confs):
    w = 0
    for j in range(1, len(path)):
        w += confs[pathway[j-1]][path[j]]['metric']
    return w

metricCol = 4
RMSDCalc = RMSDCalculator.RMSDCalculator()
cl = utilities.readClusteringObject("ClCont.pkl")
nodeFin = cl.getOptimalMetric(column=metricCol)
conf = nx.DiGraph()
nx.read_edgelist("conformationNetwork_4DAJ.edgelist", create_using=conf,
                 data=True, nodetype=int)
for source, target in conf.edges_iter():
    clusterSource = cl.getCluster(source)
    clusterTarget = cl.getCluster(target)
    conf[source][target]['metric'] = RMSDCalc.computeRMSD(clusterSource.pdb, clusterTarget.pdb)

# paths = nx.all_simple_paths(conf, 0, nodeFin)
paths = nx.shortest_simple_paths(conf, 0, nodeFin, weight='metric')
for i, path in enumerate(paths, start=1):
    if i > 50:
        break
    elif i < 3:
        cl.writePathwayTrajectory(path, "pathway_%d.pdb" % (i-1))
    print("Path ", i, "length ", weight(path, conf))



from __future__ import absolute_import, division, print_function, unicode_literals
import socket
import matplotlib
import numpy as np
import os
import collections
import argparse
machine = socket.gethostname()
if machine == "bsccv03":
    matplotlib.use('wxagg')
elif 'login' in machine:
    matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
try:
    # This might fail for older versions of matplotlib (e.g in life cluster)
    plt.style.use("ggplot")
except NameError:
    pass


def printHelp():
    """
        Create command line interface

        :returns: str -- Output filename ( if specified )
    """
    desc = "Program that prints the number of clusters throughout an adaptive sampling simulation. "\
           "It must be run in the root folder. "
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("-f", "--filename", type=str, default="", help="Output filename")
    parser.add_argument("-o", "--output", type=str, default="", help="Output folder")
    args = parser.parse_args()
    return args.filename, args.output


def getClusteringSummaryContent(summaryFile):
    """
        Get the contents of clustering summary file

        :param summaryFile: Clustering summary file
        :type summaryFile: str

        :returns: list -- List with the contents of the clustering summary file
    """
    if os.path.isfile(summaryFile):
        summaryContent = np.genfromtxt(summaryFile)
        if summaryContent.ndim > 1:
            return summaryContent
        elif summaryContent.ndim == 1:
            # file has only one line
            return np.array([summaryContent])
        else:
            # file existed but was empty
            return []
    else:
        return []


def getTotalNumberOfClustersPerEpoch(templetizedClusteringSummaryFile, folder):
    """
        Get the number of clusters in each epoch

        :param templetizedClusteringSummaryFile: Template name of the
            clustering summary file
        :type templetizedClusteringSummaryFile: str
        :param folder: Folder where the simulation data is stored
        :type folder: str

        :returns: list -- List with the number of cluster in each simulation
            epoch
    """
    allFolders = os.listdir(folder)
    numberOfEpochs = len([epoch for epoch in allFolders if epoch.isdigit() and os.path.isfile(templetizedClusteringSummaryFile % int(epoch))])

    totalNumberOfClustersPerEpoch = []
    for epoch in range(numberOfEpochs):
        clusteringSummary = getClusteringSummaryContent(templetizedClusteringSummaryFile % epoch)

        if clusteringSummary != []:
            totalNumberOfClustersPerEpoch.append(len(clusteringSummary))

    return totalNumberOfClustersPerEpoch


def findDifferentClustersInEpoch(column, summaryFile):
    """
        Get the distribution of values of a certain column in the clustering
            summary

        :param column: Column of interest
        :type column: int
        :param summaryFile: Clustering summary file
        :type summaryFile: str
        :returns: dict -- Dictionary with the set of different elements in
            column and the number of elements in this epoch
    """
    clusteringSummary = getClusteringSummaryContent(summaryFile)

    epochDictionary = {}
    if clusteringSummary != []:
        for line in clusteringSummary:
            value = line[column]
            if value not in epochDictionary:
                epochDictionary[value] = len(np.argwhere(clusteringSummary[:, column] == value))
    return epochDictionary


def findDifferentClustersForAllEpochs(column, templetizedClusteringSummaryFile, numberOfEpochs):
    """
        Get the distribution of values of a certain column in the clustering
            summary for each epoch

        :param column: Column of interest
        :type column: int
        :param templetizedClusteringSummaryFile: Template name of the
            clustering summary file
        :type templetizedClusteringSummaryFile: str
        :param numberOfEpochs: Total number of epochs in the simulation
        :type numberOfEpochs: int

        :returns: list -- List with dictionaries for all epochs. The dictionary
            has the set of different values (according to column) and their
            number
    """
    clustersPerEpoch = []
    for epoch in range(numberOfEpochs):
        summaryFile = templetizedClusteringSummaryFile % epoch
        epochDictionary = findDifferentClustersInEpoch(column, summaryFile)

        clustersPerEpoch.append(epochDictionary)
    return clustersPerEpoch


def getAllDifferentValues(clustersPerEpoch):
    """
        Get all the different values ocurring during a simulation

        :param clustersPerEpoch: List with dictionaries for all epochs. The dictionary
            has the set of different values (according to column) and their
            number
        :type clustersPerEpoch: list

        :returns: set -- Set containing all values ocurring during a simulation
    """
    allValues = set()
    for epochSummary in clustersPerEpoch:
        for value in epochSummary:
            allValues.update([value])
    return allValues


def buildClustersPerValue(clustersPerEpoch, numberOfEpochs):
    """
        Get the number of clusters that have each value

        :param clustersPerEpoch: List with dictionaries for all epochs. The dictionary
            has the set of different values (according to column) and their
            number
        :type clustersPerEpoch: list
        :param numberOfEpochs: Total number of epochs in the simulation
        :type numberOfEpochs: int

        :returns: dict -- Dictionary with the number of clusters that have each
            value
    """
    clustersPerValue = collections.defaultdict(list)

    allValues = getAllDifferentValues(clustersPerEpoch)

    for epochSummary in clustersPerEpoch:
        foundValues = set()
        for value, numClusters in epochSummary.items():
            clustersPerValue[value].append(numClusters)
            foundValues.update([value])

        for value in allValues - foundValues:
            clustersPerValue[value].append(0)

    return clustersPerValue


def getNumberOfClustersPerEpochForGivenColumn(column, templetizedClusteringSummaryFile, folder):
    """
        Get the number of clusters that have each value at each epoch

        :param column: Column of interest
        :type column: int
        :param templetizedClusteringSummaryFile: Template name of the
            clustering summary file
        :type templetizedClusteringSummaryFile: str
        :param folder: Folder where the simulation data is stored
        :type folder: str

        :returns: dict -- Dictionary with the number of clusters that have each
            value
    """
    allFolders = os.listdir(folder)
    numberOfEpochs = len([epoch for epoch in allFolders if epoch.isdigit() and os.path.isfile(templetizedClusteringSummaryFile % int(epoch))])

    clustersPerEpoch = findDifferentClustersForAllEpochs(column, templetizedClusteringSummaryFile, numberOfEpochs)

    return buildClustersPerValue(clustersPerEpoch, numberOfEpochs)


def plotClustersPerValue(clustersPerValue):
    """
        Plot the number of clusters that have a certain value

        :param clustersPerValue: Dictionary with the number of clusters that have each
            value
        :type clustersPerValue: dict
    """
    values = list(clustersPerValue.keys())
    sortedValues = np.sort(values)
    for value in sortedValues:
        plt.plot(clustersPerValue[value], label=str(value))


def plotContactsHistogram(folder, templetizedClusteringSummaryFile):
    """
        Plot the histogram of the number of contacts

        :param folder: Folder where the simulation data is stored
        :type folder: str
        :param templetizedClusteringSummaryFile: Template name of the
            clustering summary file
        :type templetizedClusteringSummaryFile: str
    """
    allFolders = os.listdir(folder)
    lastEpoch = len([epoch for epoch in allFolders if epoch.isdigit() and os.path.isfile(templetizedClusteringSummaryFile % int(epoch))]) - 1
    lastSummary = templetizedClusteringSummaryFile % lastEpoch
    contactsColumn = 3
    allContacts = np.loadtxt(lastSummary, usecols=(contactsColumn,), ndmin=1)
    plt.hist(allContacts)


def main(filename, outputPath):
    """
        Plot a summary of the clustering for a simulation:
            1) Number of clusters for each threshold value at each epoch
            2) Number of clusters for each density value at each epoch
            3) Histogram of the number of contacts
    """

    if filename:
        print("FILENAME", filename)
    outputPath = os.path.join(outputPath, "")
    if outputPath and not os.path.exists(outputPath):
        os.makedirs(outputPath)

    # Params
    clusteringFileDensityColumn = 5
    clusteringFileThresholdColumn = 4
    clusteringFolder = "clustering"
    summaryFile = "summary.txt"
    folder = "."
    # end params

    clusteringSummaryFile = os.path.join(clusteringFolder, summaryFile)
    templetizedClusteringSummaryFile = os.path.join("%d", clusteringSummaryFile)

    totalNumberOfClustersPerEpoch = getTotalNumberOfClustersPerEpoch(templetizedClusteringSummaryFile, folder)
    clustersPerDensityValue = getNumberOfClustersPerEpochForGivenColumn(clusteringFileDensityColumn, templetizedClusteringSummaryFile, folder)
    clustersPerThresholdValue = getNumberOfClustersPerEpochForGivenColumn(clusteringFileThresholdColumn, templetizedClusteringSummaryFile, folder)

    plt.figure(1)
    plt.plot(totalNumberOfClustersPerEpoch, label="All clusters")
    if filename != "":
        plt.savefig("%s%s_total.png" % (outputPath, filename))

    plotClustersPerValue(clustersPerDensityValue)
    plt.title("Number of cluser per density value")
    plt.xlabel("Epoch")
    plt.ylabel("Number of clusters")
    plt.legend(loc=2)
    if filename != "":
        plt.savefig("%s%s_density.png" % (outputPath, filename))

    plt.figure(2)
    plt.plot(totalNumberOfClustersPerEpoch, label="All clusters")
    plotClustersPerValue(clustersPerThresholdValue)
    plt.title("Number of cluser per threshold value")
    plt.xlabel("Epoch")
    plt.ylabel("Number of clusters")
    plt.legend(loc=2)
    if filename != "":
        plt.savefig("%s%s_threshold.png" % (outputPath, filename))

    plt.figure(3)
    plotContactsHistogram(folder, templetizedClusteringSummaryFile)
    plt.title("Contact ratio distribution")
    plt.xlabel("Contact ratio")
    if filename != "":
        plt.savefig("%s%s_hist.png" % (outputPath, filename))
    plt.show()

if __name__ == "__main__":
    file_name, outputFolder = printHelp()
    main(file_name, outputFolder)



from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import range
from six import reraise as raise_
import os
import sys
import shutil
import numpy as np
import string
import json
from scipy import linalg
try:
    import cPickle as pickle
except ImportError:
    import pickle
from AdaptivePELE.atomset import RMSDCalculator
from AdaptivePELE.freeEnergies import utils
import AdaptivePELE.atomset.atomset as atomset


class UnsatisfiedDependencyException(Exception):
    __module__ = Exception.__module__


def cleanup(tmpFolder):
    """
        Remove folder if exists

        :param tmpFolder: Folder to remove
        :type tmpFolder: str
    """
    if os.path.exists(tmpFolder):
        shutil.rmtree(tmpFolder)


def makeFolder(outputDir):
    """
        Makes folder

        :param outputDir: Folder filename
        :type outputDir: str
    """
    if not os.path.exists(outputDir):
        os.makedirs(outputDir)


def getSnapshots(trajectoryFile, verbose=False):
    """
        Gets the snapshots

        :param trajectoryFile: Trajectory filename
        :type trajectoryFile: str
        :param verbose: Add verbose to snapshots
        :type verbose: bool

        :returns: str -- Snapshots with information
    """
    with open(trajectoryFile, "r") as inputFile:
        inputFileContent = inputFile.read()

    snapshots = inputFileContent.split("ENDMDL")
    if len(snapshots) > 1:
        snapshots = snapshots[:-1]
    if not verbose:
        return snapshots

    remarkInfo = "REMARK 000 File created using PELE++\nREMARK source            : %s\nREMARK original model nr : %d\nREMARK First snapshot is 1, not 0 (as opposed to report)\n"
    snapshotsWithInfo = [remarkInfo % (trajectoryFile, i+1)+snapshot for i, snapshot in enumerate(snapshots)]
    return snapshotsWithInfo


def getTrajNum(trajFilename):
    """
        Gets the trajectory number

        :param trajFilename: Trajectory filename
        :type trajFilename: str

        :returns: int -- Trajectory number
    """
    return int(trajFilename.split("_")[-1][:-4])


def calculateContactMapEigen(contactMap):
    """
        Calculates eigenvectors and values of an extended contact map

        :param contactMap: Contact map
        :type contactMap: np.array

        :returns: (numpy.ndarray, numpy.ndarray) -- eigenvalues, eigenvectors
    """
    nLig, nCA = contactMap.shape
    extendedCM = np.zeros((nLig+nCA, nLig+nCA))
    extendedCM[nCA:, :nCA] = contactMap
    extendedCM[:nCA, nCA:] = contactMap.T
    assert (extendedCM == extendedCM.T).all(), "Extended ContactMap not symmetric"
    eiv, eic = np.linalg.eigh(extendedCM)
    return eiv, eic


def assertSymmetriesDict(symmetries, PDB):
    """
        Asserts the symmetry list in a PDB

        :param symmetries: List of symmetry groups
        :type symmetries: list of str
        :param PDB: PDB object to check symmetry list against
        :type PDB: :py:class:`.PDB`

        :raise AssertionError: If an atom is not found in the structure
    """
    for group in symmetries:
        for key in group:
            assert key in PDB.atoms, "Symmetry atom %s not found in initial structure" % key
    if symmetries:
        print("Symmetry dictionary correctly defined!")


def getRMSD(traj, nativePDB, resname, symmetries):
    """
        Computes the RMSD of a trajectory, given a native and symmetries

        :param traj: Trajecotry filename
        :type traj: str
        :param nativePDB:  Native PDB object
        :type native PDB: :py:class:`.PDB`
        :param resname: Resname to compute its RMSD
        :type resname: str
        :param symmetries: Symmetries dictionary list with independent symmetry groups
        :type symmetries: list of dict

        :return: np.array -- Array with the rmsd values of the trajectory
    """

    snapshots = getSnapshots(traj)
    rmsds = np.zeros(len(snapshots))
    RMSDCalc = RMSDCalculator.RMSDCalculator(symmetries)
    for i, snapshot in enumerate(snapshots):
        snapshotPDB = atomset.PDB()
        snapshotPDB.initialise(snapshot, resname=resname)

        rmsds[i] = RMSDCalc.computeRMSD(nativePDB, snapshotPDB)

    return rmsds


def readClusteringObject(clusteringObjectPath):
    """
        Reads and returns a clustering object

        :param clusteringObjectPath: Clustering object path
        :type clusteringObjectPath: str

        :raise EOFError: If the object is empty

        :returns: :py:class:`.Clustering` -- clusteringObject
    """
    with open(clusteringObjectPath, 'rb') as f:
        try:
            return pickle.load(f)
        except EOFError:
            t, v, tb = sys.exc_info()
            raise_(t, v, tb)


def ensure_connectivity_msm(msm):
    if msm.nstates_full == msm.nstates:
        return msm.stationary_distribution
    else:
        counts = msm.count_matrix_full
        counts += 1/counts.shape[0]
        trans = utils.buildRevTransitionMatrix(counts)
        _, eic = getSortedEigen(trans)
        return getStationaryDistr(eic[:, 0])


def getStationaryDistr(lowestEigenvector):
    absStationary = np.abs(lowestEigenvector)
    return absStationary / absStationary.sum()


def getSortedEigen(T):
    eigenvals, eigenvectors = linalg.eig(T, left=True, right=False)
    sortedIndices = np.argsort(eigenvals)[::-1]

    # reigenvals, reigenvectors = linalg.eig(T.T)
    # rsortedIndices = np.argsort(reigenvals)[::-1]
    return eigenvals[sortedIndices], eigenvectors[:, sortedIndices]


def get_epoch_folders(path):
    allFolders = os.listdir(path)
    folders = [epoch for epoch in allFolders if epoch.isdigit()]
    folders.sort(key=int)
    return folders


def gen_atom_name(index):
    # 6760 = 26*26*10
    # The maximum number of unique ids generated by this method is 175760 =
    # 26*26*26*10, however, remember that generally the maximum number of unique
    # atoms that can be used with vmd is around 32000
    ind1 = index//6760
    ind2 = (index % 6760)
    ind3 = ind2 % 260
    return chr(65+ind1)+chr(65+ind2//260)+chr(65+ind3//10)+str(ind3 % 10)


def write_PDB_clusters(pmf_xyzg, title="clusters.pdb", use_beta=False, elements=None):
    templateLine = "HETATM%s %s CLT L 502    %s%s%s  0.75%s          %s  \n"
    if elements is None:
        elements = ["H" for i in range(len(pmf_xyzg))]

    content = ""
    names = []
    for i, line in enumerate(pmf_xyzg):
        number = str(i).rjust(5)
        # number3 = str(i).ljust(3)
        number3 = gen_atom_name(i).ljust(4)
        names.append(number3)
        x = ("%.3f" % line[0]).rjust(8)
        y = ("%.3f" % line[1]).rjust(8)
        z = ("%.3f" % line[2]).rjust(8)
        element = elements[i].rjust(2)
        if use_beta:
            g = ("%.2f" % line[-1]).rjust(6)
        else:
            g = ("%.2f" % 0).rjust(6)
        content += templateLine % (number, number3, x, y, z, g, element)

    with open(title, 'w') as f:
        f.write(content)
    return names


def distanceCOM(coords1, coords2):
    coords1 = np.array(coords1)
    coords2 = np.array(coords2)
    return np.linalg.norm(coords1-coords2)


def sign(x, tol=1e-7):
    """
        Return the sign of a number

        :param x: Array of number to evaluate the sign
        :type x: numpy.array
        :param tol: Tolerance to define the zero
        :type tol: float

        :returns: int -- Sign of the number
    """
    x[abs(x) < tol] = 0
    return np.sign(x)


def getAtomNames(values):
    """
        Assign to each value an atomname, O for negatives, H for 0 and N for
        positives. This function is created for assign atomnames for custom pdbs

        :param values: Collection of numbers to assing an atom name for
        :type values: iterable

        :returns: list -- List of atom names
    """
    names = ["O", "H", "N"]
    values += 1
    return [names[int(value)] for value in values]


def getReportAndTrajectoryWildcard(JSONdict):
    """
        Extract the trajectory and report filename from the pele control file

        :param JSONdict: Dictionary containing a parsed PELE control file
        :type JSONdict: dict

        :returns: str, str -- Report and trajectory wildcards
    """
    reportWildcard = os.path.split(JSONdict["commands"][0]["PELE_Output"]['reportPath'])[1]
    trajWildcard = os.path.split(JSONdict["commands"][0]["PELE_Output"]['trajectoryPath'])[1]
    trajWildcard = '_%d'.join(os.path.splitext(trajWildcard))
    reportWildcard = '_%d'.join(os.path.splitext(reportWildcard))
    return reportWildcard, trajWildcard


def getPELEControlFileDict(templetizedControlFile):
    """
        Parse a PELE control file into a python dictionary
    """

    with open(templetizedControlFile) as fc:
        peleControlFile = fc.read()

    templateNames = {ele[1]: '"$%s"' % ele[1] for ele in string.Template.pattern.findall(peleControlFile)}
    templateNames.pop("OUTPUT_PATH", None)
    return json.loads(string.Template(peleControlFile).safe_substitute(templateNames)), templateNames


def getMetricsFromReportsInEpoch(reportName, outputFolder, nTrajs):
    """
        Extract the metrics in report file from an epoch to a numpy array
    """
    metrics = []
    for i in range(1, nTrajs):
        report = np.loadtxt(os.path.join(outputFolder, reportName % i))
        if len(report.shape) < 2:
            metrics.append(report.tolist()+[i, 0])
        else:
            traj_line = np.array([i] * report.shape[0])
            snapshot_line = np.array(range(report.shape[0]))
            metrics.extend(np.hstack((report, traj_line[:, np.newaxis], snapshot_line[:, np.newaxis])))
    return np.array(metrics)


def getSASAcolumnFromControlFile(JSONdict):
    for i, metricBlock in enumerate(JSONdict["commands"][0]["PeleTasks"][0]['metrics']):
        if 'sasa' in metricBlock['type'].lower():
            # Report files have 4 fixed columnts, task, steps, accepted steps
            # and energy
            return i+4
    raise ValueError("No SASA metric found in control file!!! Please add it in order to use the moving box feature")



from AdaptivePELE.constants import blockNames


class SPAWNING_TYPES:
    sameWeight, inverselyProportional, epsilon, simulatedAnnealing, FAST, variableEpsilon, UCB, independent, REAP, null = list(range(10))

SPAWNING_TYPE_TO_STRING_DICTIONARY = {
    SPAWNING_TYPES.independent: blockNames.StringSpawningTypes.independent,
    SPAWNING_TYPES.sameWeight: blockNames.StringSpawningTypes.sameWeight,
    SPAWNING_TYPES.inverselyProportional: blockNames.StringSpawningTypes.inverselyProportional,
    SPAWNING_TYPES.epsilon: blockNames.StringSpawningTypes.epsilon,
    SPAWNING_TYPES.FAST: blockNames.StringSpawningTypes.fast,
    SPAWNING_TYPES.variableEpsilon: blockNames.StringSpawningTypes.variableEpsilon,
    SPAWNING_TYPES.simulatedAnnealing: blockNames.StringSpawningTypes.simulatedAnnealing,
    SPAWNING_TYPES.UCB: blockNames.StringSpawningTypes.UCB,
    SPAWNING_TYPES.REAP: blockNames.StringSpawningTypes.REAP,
    SPAWNING_TYPES.null: blockNames.StringSpawningTypes.null
}


class EPSILON_VARIATION_TYPES:
    linearVariation, contactsVariation = list(range(2))

EPSILON_VARIATION_TYPE_TO_STRING_DICTIONARY = {
    EPSILON_VARIATION_TYPES.linearVariation: blockNames.VariableEpsilonTypes.linearVariation,
    EPSILON_VARIATION_TYPES.contactsVariation: blockNames.VariableEpsilonTypes.contactsVariation,
}



from __future__ import absolute_import, division, print_function, unicode_literals
import glob
import os
import argparse
import matplotlib
import matplotlib.pyplot as plt
import numpy as np
from scipy import linalg

FOLDER = "discretized"
CLUSTER_CENTERS = "clusterCenters.dat"
TRAJECTORY_MATCHING_PATTERN = "*.disctraj"


def parseArguments():
    desc = "Program that analyses a column of data, printing different statistical values and a histogram if desired."
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("-f", default='discretized', help="Folder with cluster contents")
    parser.add_argument("-t", "--threshold", default=0, type=float, help="If Cij < threshold and Cji < threshold, Cij = Cji = 0")
    parser.add_argument("-l", "--lagtime", default=1, type=int, help="Lagtime")
    parser.add_argument("-p", "--printFigs", action="store_true")
    args = parser.parse_args()

    return args.f, args.threshold, args.lagtime, args.printFigs


def getNumberOfClusters(folder_name):
    clusterCentersFilename = os.path.join(folder_name, CLUSTER_CENTERS)
    clusterCentersFile = open(clusterCentersFilename, 'r')
    clusterCenters = clusterCentersFile.readlines()
    clusterCentersFile.close()
    return len(clusterCenters)


def normalizePopulations(populations):
    return populations / populations.sum()


def normalizeTransitions(transitions):
    transitionsSum = transitions.sum(axis=1, dtype='float')
    transitionsSum = np.transpose(transitionsSum)
    normalizedPopulations = np.divide(transitions, transitionsSum[:, np.newaxis])
    return normalizedPopulations


def computeCountsAndCountMatrix(trajectories, numberOfClusters, lag_time=1):

    transitions = np.zeros((numberOfClusters, numberOfClusters))
    populations = np.zeros(numberOfClusters)

    for trajectoryFilename in trajectories:
        dtraj = np.loadtxt(trajectoryFilename, dtype=int, ndmin=1)
        # can be done much faster with sparse matrices (see runMarkovChain script)
        for i in range(len(dtraj) - lag_time):
            fromCluster = dtraj[i]
            toCluster = dtraj[i + lag_time]
            transitions[fromCluster][toCluster] += 1
        populations[dtraj] += 1

    return populations, transitions


def removeNoise(countMatrix, threshold):
    for i in range(len(countMatrix)):
        for j in range(i+1, len(countMatrix[i])):
            if countMatrix[i][j] < threshold or countMatrix[j][i] < threshold:
                countMatrix[i][j] = 0
                countMatrix[j][i] = 0
    return countMatrix


def computeCountsAndCountMatrixRemovingNoise(folder_name, countsThreshold, lag_time):
    """
        Computes the number of counts per cluster (#times it's been visited)
        and the count matrix, setting to 0 those counts below the threshold
    """
    trajectoryMatchingPattern = os.path.join(folder_name, TRAJECTORY_MATCHING_PATTERN)
    trajectories = glob.glob(trajectoryMatchingPattern)

    nclusters = getNumberOfClusters(folder_name)
    countsPerCluster, countMatrix = computeCountsAndCountMatrix(trajectories, nclusters, lag_time)

    countMatrixWithoutNoise = removeNoise(countMatrix, countsThreshold)
    countMatrixWithoutNoise += 1./nclusters  # added pseudo counts to avoid nans in relative entropy calc

    return countsPerCluster, countMatrixWithoutNoise


def computePopulationsAndTransitionProbabilities(folder, countsThreshold, lagtime):
    counts, countMatrix = computeCountsAndCountMatrixRemovingNoise(folder, countsThreshold, lagtime)

    # countMatrix = np.array(transitions)
    # sparseCountMatrix = sparse.csr_matrix(countMatrix)
    # sparseInString = sparseCountMatrix.__str__()
    # outputFile = open("countMatrix.dat", "w")
    # outputFile.write(sparseInString)
    # outputFile.close()

    populations = normalizePopulations(counts)
    transitions = normalizeTransitions(countMatrix)

    # for i in range(len(transitions)):
    #     for j in range(i+1, len(transitions[i])):
    #         if transitions[i][j] < probabilityThershold and transitions[j][i] < probabilityThershold:
    #             transitions[i][j] = 0
    #             transitions[j][i] = 0

    return populations, transitions


def plotMatrix(figureNumber, titleString, matrix, cmap=''):
    fig = plt.figure(figureNumber)
    fig.set_facecolor('white')
    ax = fig.add_subplot(111)
    ax.set_aspect('equal')
    plt.title(titleString, fontsize=20)
    if cmap != '':
        cax = plt.pcolor(matrix, alpha=0.9, cmap=cmap)
    else:
        cax = plt.pcolor(matrix, alpha=0.9)

    # add colorbar if does not exist
    # if exists, remove previous, and add new one
    if len(plt.gcf().axes) > 1:
        # if so, then the last axes must be the colorbar.
        # we get its extent
        pts = plt.gcf().axes[-1].get_position().get_points()
        # and its label
        label = plt.gcf().axes[-1].get_ylabel()
        # and then remove the axes
        plt.gcf().axes[-1].remove()
        # then we draw a new axes a the extents of the old one
        cax = plt.gcf().add_axes([pts[0][0], pts[0][1], pts[1][0]-pts[0][0], pts[1][1]-pts[0][1]])
        # and add a colorbar to it
        cbar = plt.colorbar(cax=cax)
        cbar.ax.set_ylabel(label)
        # unfortunately the aspect is different between the initial call to colorbar
        #   without cax argument. Try to reset it (but still it's somehow different)
        cbar.ax.set_aspect(20)
    else:
        plt.colorbar(cax)


def barPlot(figureNumber, titleString, array):
    fig = plt.figure(figureNumber)
    fig.set_facecolor('white')
    ax = fig.add_subplot(111)
    plt.title(titleString)
    ax.bar(np.arange(len(array)), array, width=1, alpha=0.4, color='b')


def getRelativeEntropy(goldenStationary, goldenT, T):
    return np.dot(goldenStationary, goldenT*np.log(goldenT/T)).sum()


def main(folder, countsThreshold, lagtime, printFigs=False):
    populations, transitions = computePopulationsAndTransitionProbabilities(folder, countsThreshold, lagtime)

    if printFigs:
        cmap = matplotlib.cm.coolwarm
        cmap.set_bad('w', 1.)
        matplotlib.rc('font', family='Times New Roman')

    if printFigs:
        # transitionsWithoutDiagonal = np.copy(transitions)
        # for i in range(len(transitions)):
        #     transitionsWithoutDiagonal[i][i] = 0
        # plotMatrix(6, r'$P_{ij}, P_{ii} = 0 \forall i$', transitionsWithoutDiagonal,cmap)
        pass

    # p_i * P_ij
    detailedBalanceComponents = np.copy(transitions)
    for i in range(len(detailedBalanceComponents)):
        detailedBalanceComponents[i] = np.multiply(populations[i], transitions[i])

    if printFigs:
        # barPlot(1, 'Population', populations)

        # plotMatrix(2, r'$P_{ij}$', transitions, cmap)
        plotMatrix(3, r'$\pi_i P_{ij}$', detailedBalanceComponents, cmap)
        # plt.savefig("db_flux.eps")

    detailedBalanceComponentsAbsoluteDifference = np.absolute(detailedBalanceComponents - detailedBalanceComponents.T) / 2.  # factor 2 to avoid the metric to go from 0 to 2, but from 0 to 1
    detailedBalanceComponentsAverage = np.multiply(detailedBalanceComponents + detailedBalanceComponents.T, 0.5)
    frobeniusAvg = linalg.norm(detailedBalanceComponentsAbsoluteDifference) / linalg.norm(detailedBalanceComponentsAverage)
    print("|semidiff| / |average|", frobeniusAvg)

    np.seterr(divide='ignore', invalid='ignore')
    detailedbalanceComponentsRelativeDifference = np.divide(detailedBalanceComponentsAbsoluteDifference, detailedBalanceComponentsAverage)
    maskedDetailedbalanceComponentsRelativeDifference = np.ma.array(detailedbalanceComponentsRelativeDifference, mask=np.isnan(detailedbalanceComponentsRelativeDifference))

    if printFigs:
        plotMatrix(4, r'$|F_{ij} - F_{ji}|$', detailedBalanceComponentsAbsoluteDifference, cmap)
        plt.savefig("db_abs_diff.eps")
        plotMatrix(5, r'$M(F)$', maskedDetailedbalanceComponentsRelativeDifference, cmap)
        plt.savefig("db_frobenius.eps")

    return frobeniusAvg


if __name__ == '__main__':
    folder_path, counts_Threshold, lagTime, print_Figs = parseArguments()
    main(folder_path, counts_Threshold, lagTime, print_Figs)



from __future__ import absolute_import, division, print_function, unicode_literals
from AdaptivePELE.validator import validatorBlockNames
from AdaptivePELE.constants import blockNames
import json
import warnings
import numbers
import sys
import ast
try:
    # Check if the basestring type if available, this will fail in python3
    basestring
except NameError:
    basestring = str


def validate(control_file):
    """
        Validate an AdaptivePELE control file to ensure that there are no errors

        :param controlFile: Adaptive sampling control file
        :type controlFile: str

        :raise ValueError: If a error is found in the control file
    """
    isCorrect = True
    with open(control_file, 'r') as f:
        jsonFile = f.read()
    try:
        parsedJSON = json.loads(jsonFile)
    except ValueError:
        print(jsonFile)
        raise ValueError("Invalid JSON file!")

    for block in dir(validatorBlockNames.ControlFileParams):
        if block.startswith('__'):
            continue
        block_obj = getattr(validatorBlockNames,
                            eval("validatorBlockNames.ControlFileParams.%s" %
                                                         block))
        controlfile_obj = eval("blockNames.ControlFileParams.%s" % block)

        if block == "generalParams":
            try:
                blockCorrect = validateGeneralBlock(block_obj,
                                                    parsedJSON[controlfile_obj])
                isCorrect = isCorrect and blockCorrect
            except KeyError:
                warnings.warn("Block %s not found in control file!" %
                              controlfile_obj)
                isCorrect = False
        else:
            try:
                blockCorrect = validateBlock(block_obj,
                                             parsedJSON[controlfile_obj])
                isCorrect = isCorrect and blockCorrect
            except KeyError:
                warnings.warn("Block %s not found in control file!" %
                              controlfile_obj)
                isCorrect = False

    if isCorrect:
        print("Congratulations! No errors found in your control file!")
    else:
        print(jsonFile)
        raise ValueError("There are errors in your control file!!!")
    return True


def validateBlock(blockName, controlFileBlock):
    """
        Validate the a block of the control file to ensure that
        there are no errors. Raise a warning if an error is found.

        :param blockName: Dictionary containing the parameters and possible
            types for the block
        :type blockName: dict
        :param controlFileBlock: Dictionary containing the parameters specified
            in the control file for the block
        :type controlFileBlock: dict

        :returns: bool -- Wheter and error has been found in a block
            block
    """
    isCorrect = True
    try:
        blockType = controlFileBlock["type"]
    except KeyError:
        isCorrect = False
        warnings.warn("Missing mandatory parameter type in block")

    # Check if type selected is valid
    if not isinstance(blockType, basestring):
        warnings.warn("Type for %s should be %s and instead is %s" %
                      (blockType, 'str', type(blockType).__name__))
        isCorrect = False

    # check for mandatory parameters
    try:
        for mandatory, value in blockName.types[blockType].items():
            try:
                if not isinstance(controlFileBlock['params'][mandatory], eval(value)):
                    warnings.warn("Type for %s should be %s and instead is %s" %
                                  (mandatory, value, type(controlFileBlock['params'][mandatory]).__name__))
                    isCorrect = False
            except KeyError as err:
                warnings.warn("%s missing: Mandatory parameter %s in %s not found." %
                              (str(err), mandatory, blockName.__name__))
                isCorrect = False
    except KeyError as err:
        warnings.warn("Missing %s: Type %s in %s not found." %
                      (str(err), blockType, blockName.__name__))
        isCorrect = False
    # check rest of parameters specified
    try:
        for param, value in controlFileBlock["params"].items():
            try:
                if not isinstance(value, eval(blockName.params[param])):
                    warnings.warn("Type for %s should be %s and instead is %s" %
                                  (param, blockName.params[param],
                                   type(value).__name__))
                    isCorrect = False
            except KeyError:
                warnings.warn("Parameter %s in block %s not recognized." %
                              (param, blockName.__name__))
                isCorrect = False
    except KeyError as err:
        warnings.warn("Missing %s in %s" % (str(err), blockName.__name__))
        isCorrect = False

    for block in dir(blockName):
        if not block.startswith('__') and block not in ["params", "types"]:
            # The parameters blocks for density and threshold calculator are
            # not mandatory
            if block not in controlFileBlock:
                continue
            try:
                types_dict = eval("blockName.%s" % block)["types"]
                params_dict = eval("blockName.%s" % block)["params"]
            except KeyError as err:
                warnings.warn("Type %s in %s not found." %
                              (str(err), block))
                isCorrect = False
            try:
                blockType = controlFileBlock[block]["type"]
            except KeyError:
                isCorrect = False
                warnings.warn("Missing mandatory parameter type in block %s" %
                              block)
            if blockType not in types_dict:
                warnings.warn("Type %s in %s not found." %
                              (blockType, blockName.__name__))
                isCorrect = False
            if not isinstance(blockType, basestring):
                warnings.warn("Type for %s should be %s and instead is %s" %
                              (blockType, 'str', type(blockType).__name__))
                isCorrect = False
            # check rest of parameters specified
            # Do a get on the "params" block and return an empty list if not found
            paramsControlFile = controlFileBlock[block].get("params", {})
            for param, value in paramsControlFile.items():
                try:
                    if not isinstance(value, eval(params_dict[param])):
                        warnings.warn("Type for %s should be %s and instead is %s" %
                                      (param, params_dict[param], type(value).__name__))
                        isCorrect = False
                except KeyError:
                    warnings.warn("Parameter %s not recognized." % param)
                    isCorrect = False

    return isCorrect


def validateGeneralBlock(blockName, controlFileBlock):
    """
        Validate the generalParams block of the control file to ensure that
        there are no errors. Raise a warning if an error is found.

        :param blockName: Dictionary containing the parameters and possible
            types for the block
        :type blockName: dict
        :param controlFileBlock: Dictionary containing the parameters specified
         in the control file for the block
        :type controlFileBlock: dict

        :returns: bool -- Wheter and error has been found in the generalParams
            block
    """
    isCorrect = True
    for key, value in blockName.mandatory.items():
        try:
            if not isinstance(controlFileBlock[key], eval(value)):
                warnings.warn("Type for %s should be %s and instead is %s" %
                              (key, value, type(controlFileBlock[key]).__name__))
                isCorrect = False
        except KeyError:
            warnings.warn("Mandatory parameter %s in GeneralParams not found." %
                          key)
            isCorrect = False

    for key, value in controlFileBlock.items():
        try:
            if not isinstance(value, eval(blockName.params[key])):
                warnings.warn("Type for %s should be %s and instead is %s" %
                              (key, value, type(blockName.params[key]).__name__))
                isCorrect = False
        except KeyError:
            warnings.warn("Parameter %s in GeneralParams not recognized." %
                          key)
            isCorrect = False
    return isCorrect

if __name__ == "__main__":

    if len(sys.argv) > 1:
        validate(sys.argv[1])
    else:
        controlFiles = ["tests/data/3ptb_data/integrationTest%i.conf" % i for i in range(1, 4)]
        for contfile in controlFiles:
            print("Validating control file %s" % contfile)
            validate(contfile)



# -*- coding: utf-8 -*-

import sys
import re
from frag_pele.PlopRotTemp_S_2017.template.tmp_helpers import Helper



class ChargeHandler(Helper):

    """
        Handler class in charge of retrieving
        atom charges from file to TemplateBuilder
    """

    def __init__(self, file):
        self.file = file
        

    def get_charges(self):
        """
            Parse self.file for charges and retrieve.
        """
        charges = []

        with open(self.file, "r") as f:
            while f:  # Find Bond Section
                line = f.readline()
                if line.startswith(" m_atom"):
                    break
            keywords = []
            while f:  # Read keywords                
                line = f.readline()
                if (re.search(':::', line)):
                    break
                else:
                    keyword = "index" if "index" in line else line.strip()
                    keywords.append(keyword)
            while f:  # get charges
                line = f.readline()
                if not re.search(':::', line):
                    line = re.sub(' +',' ',line).strip('\n').strip().split()
                    for i, keyword in enumerate(keywords):
                        if keyword == "r_m_charge1":          
                            charge = line[i]
                            charges.append(charge)
                else:
                    if charges: return charges
                    else: raise ValueError("NO CHARGES IN MAE")
            

                    




    



from __future__ import absolute_import, division, print_function, unicode_literals
import argparse
from AdaptivePELE.atomset import atomset, SymmetryContactMapEvaluator
from AdaptivePELE.utilities import utilities
import matplotlib.pyplot as plt


def parseArguments():
    desc = "Calculate the histogram of ContactMap contacts over a trajectory, either as pdb files or from a clustering\n"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("resname", type=str, help="Ligand resname in pdb")
    parser.add_argument("contactThreshold", type=int, help="Contact threshold to calculate contactMap")
    parser.add_argument("-trajectory", type=str, nargs='+', help="Path to the trajectory or pdbs to analyse")
    parser.add_argument("-clustering", help="Path to the clustering object to analyse")
    parser.add_argument("-nRes", type=int, default=10, help="Number of top residues to display")
    args = parser.parse_args()
    return args.trajectory, args.clustering, args.nRes, args.resname, args.contactThreshold


def generateConformations(resname, clAcc, trajectory):
    if clAcc is None:
        for traj in trajectory:
            snapshots = utilities.getSnapshots(traj)
            for snapshot in snapshots:
                PDBobj = atomset.PDB()
                PDBobj.initialise(snapshot, resname)
                yield PDBobj
    else:
        for cluster in clAcc.clusters.clusters:
            yield cluster.pdb

if __name__ == "__main__":
    traj_name, clustering, nRes, lig_resname, contactThreshold = parseArguments()

    if clustering is None:
        clusterAcc = None
    else:
        clusetrAcc = utilities.readClusteringObject(clustering)

    totalAcc = []
    symEval = SymmetryContactMapEvaluator.SymmetryContactMapEvaluator()
    refPDB = None

    for pdb in generateConformations(lig_resname, clusetrAcc, traj_name):
        if refPDB is None:
            refPDB = pdb
        contactMap, foo = symEval.createContactMap(pdb, lig_resname, contactThreshold)
        if len(totalAcc):
            totalAcc += contactMap.sum(axis=0, dtype=bool).astype(int)
        else:
            totalAcc = contactMap.sum(axis=0, dtype=bool).astype(int)

    proteinList = symEval.proteinList
    residueCounts = {}
    totCounts = 0
    for atomID, counts in zip(proteinList, totalAcc):
        res = refPDB.atoms[atomID].resnum
        totCounts += counts
        if res in residueCounts:
            residueCounts[res] += counts
        else:
            residueCounts[res] = counts

    for res in residueCounts:
        residueCounts[res] /= float(totCounts)

    print("Residue\tResidue frequency")
    for res in sorted(residueCounts, key=lambda x: residueCounts[x], reverse=True)[:nRes]:
        print("%s\t%.4f" % (res, residueCounts[res]))

    plt.figure()
    plt.ylabel("Contact frequency")
    plt.xlabel("Residue number")
    plt.bar(list(residueCounts.keys()), residueCounts.values())
    plt.savefig("hist_CM.png")
    plt.show()



from __future__ import absolute_import, division, print_function, unicode_literals
import argparse
import sys
from AdaptivePELE.atomset import atomset, SymmetryContactMapEvaluator
from AdaptivePELE.utilities import utilities
import numpy as np
import itertools
try:
    import networkx as nx
except ImportError:
    raise ImportError("Package networkx not found!!! Networkx is necessary to run this script")


def parseArguments():
    desc = "Create a network of residues that share contacts with the same ligand atom over a trajectory, either as pdb files or from a clustering\n"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("resname", type=str, help="Ligand resname in pdb")
    parser.add_argument("contactThreshold", type=int, help="Contact threshold to calculate contactMap")
    parser.add_argument("-trajectory", type=str, nargs='+', help="Path to the trajectory or pdbs to analyse")
    parser.add_argument("-clustering", help="Path to the clustering object to analyse")
    parser.add_argument("-nRes", type=int, default=10, help="Number of top residues to display")
    args = parser.parse_args()
    return args.trajectory, args.clustering, args.nRes, args.resname, args.contactThreshold


def generateConformations(resname, clAcc, trajectory):
    if clAcc is None:
        for traj in trajectory:
            snapshots = utilities.getSnapshots(traj)
            for snapshot in snapshots:
                PDBobj = atomset.PDB()
                PDBobj.initialise(snapshot, resname)
                yield PDBobj
    else:
        for cluster in clAcc.clusters.clusters:
            yield cluster.pdb


if __name__ == "__main__":
    trajectory, clustering, nRes, resname, contactThreshold = parseArguments()

    if clustering is None:
        clAcc = None
    else:
        clAcc = utilities.readClusteringObject(clustering)

    symEval = SymmetryContactMapEvaluator.SymmetryContactMapEvaluator()
    refPDB = None
    network = nx.Graph()
    sys.stderr.write("Creating network...\n")

    for pdb in generateConformations(resname, clAcc, trajectory):
        contactMap, foo = symEval.createContactMap(pdb, resname, contactThreshold)
        if refPDB is None:
            refPDB = pdb
            proteinList = symEval.proteinList
        for row in contactMap:
            nodes = np.where(row)[0]
            for pair in itertools.combinations(nodes, 2):
                atom1 = refPDB.atoms[proteinList[pair[0]]]
                atom2 = refPDB.atoms[proteinList[pair[1]]]
                res1, res2 = int(atom1.resnum), int(atom2.resnum)
                if abs(res1-res2) > 5:
                    if network.has_edge(res1, res2):
                        network[res1][res2]['weight'] += 1
                    else:
                        network.add_edge(res1, res2, weight=1)

    numAtoms = min(nRes, network.order())
    deg = network.degree()
    print("Residue\tDegree")
    for node in sorted(network.nodes(), key=lambda x: deg[x], reverse=True)[:nRes]:
        print("%d\t%d" % (node, deg[node]))
    nx.write_weighted_edgelist(network, "newtorkCM.edgelist")



import os
import sys
import argparse
import glob
from AdaptivePELE.utilities import utilities


def parseArguments():
    desc = "Write the information related to the conformation network to file\n"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("clusteringObject", type=str, help="Path to the clustering object")
    parser.add_argument("epoch", type=str, help="Path to the epoch to search the snapshot")
    parser.add_argument("trajectory", type=int, help="Trajectory number")
    parser.add_argument("snapshot", type=int, help="Snapshot to select (in accepted steps)")
    parser.add_argument("-o", type=str, default=None, help="Output path where to write the files")
    args = parser.parse_args()
    return args.clusteringObject, args.trajectory, args.snapshot, args.epoch, args.o


if __name__ == "__main__":
    clusteringObject, trajectory, snapshot, epoch, outputPath = parseArguments()
    if outputPath is not None:
        outputPath = os.path.join(outputPath, "")
        if not os.path.exists(outputPath):
            os.makedirs(outputPath)
    else:
        outputPath = ""
    sys.stderr.write("Reading clustering object...\n")
    cl = utilities.readClusteringObject(clusteringObject)
    if cl.conformationNetwork is None:
        sys.exit("Clustering object loaded has no conformation network!!")
    conf = cl.conformationNetwork
    filename = glob.glob(epoch+"/*traj*_%d.pdb" % trajectory)
    snapshots = utilities.getSnapshots(filename[0])
    snapshots = snapshots[:snapshot+1]
    procMapping = open(os.path.join(epoch, "processorMapping.txt")).read().rstrip().split(',')
    leaf = procMapping[trajectory-1]
    pathway = conf.createPathwayToCluster(int(leaf))
    cl.writePathwayTrajectory(pathway, outputPath+"pathway.pdb")
    with open(outputPath+"pathway.pdb", "a") as f:
        f.write("ENDMDL\n".join(snapshots))



from __future__ import absolute_import, division, print_function, unicode_literals
import os
import numpy as np
import glob
import pyemma.coordinates as coor
from pyemma.coordinates.clustering import AssignCenters
import scipy


class Cluster:
    def __init__(self, numClusters, trajectoryFolder, trajectoryBasename, stride=1, alwaysCluster=True):
        """
            alwaysCluster: clusterize regardless of whether discretized/clusterCenters.dat exists or not
        """

        self.discretizedFolder = "discretized"
        self.clusterCentersFile = os.path.join(self.discretizedFolder, "clusterCenters.dat")
        self.clusterCenters = np.array([])
        self.dTrajTemplateName = os.path.join(self.discretizedFolder, "%s.disctraj")
        self.clusteringFile = "clustering_object.pkl"
        self.stride = stride
        self.trajFilenames = []
        self.dtrajs = []
        self.alwaysCluster = alwaysCluster
        self.numClusters = numClusters
        self.trajectoryFolder = trajectoryFolder
        self.trajectoryBasename = trajectoryBasename
        self.x = []

    def cluster(self, trajectories):
        """ Cluster the trajectories into numClusters clusters using kmeans
        algorithm.
        Returns a KmeansClusteringObject
        """
        return coor.cluster_kmeans(data=trajectories, k=self.numClusters, max_iter=500, stride=self.stride)

    def assignNewTrajectories(self, trajs):
        # wrap the clusterCentersFile argument in a str call to pass pyemma
        # assign check if isinstance of str
        assign = AssignCenters(str(self.clusterCentersFile))
        dTrajs = assign.assign(trajs)
        return dTrajs

    def clusterTrajectories(self):
        print("Loading trajectories...")
        self.x, self.trajFilenames = loadTrajFiles(self.trajectoryFolder, self.trajectoryBasename)

        # cluster & assign
        if self.alwaysCluster or not os.path.exists(self.clusterCentersFile):
            print("Clustering data...")
            cl = self.cluster(self.x)  # cl: pyemma's clusteringObject
            makeFolder(self.discretizedFolder)
            self.clusterCenters = cl.clustercenters
            self._writeClusterCenters(self.clusterCenters, self.clusterCentersFile)
            print("Assigning data...")
            self.dtrajs = cl.dtrajs[:]
        else:
            print("Assigning data (clustering exists)...")
            self.clusterCenters = np.loadtxt(self.clusterCentersFile)
            self.dtrajs = self.assignNewTrajectories(self.x)

        print("Writing clustering data...")
        self._writeDtrajs(self.trajFilenames, self.dtrajs, self.dTrajTemplateName)

    def eliminateLowPopulatedClusters(self, clusterCountsThreshold, tau=None):
        if self.dtrajs == []:
            print("Call clusterTrajectories() first!")
            return

        dtrajs = np.array(self.dtrajs).copy()
        try:
            dtrajs = np.concatenate(dtrajs[:, :-tau])
        except:
            dtrajs = np.concatenate(dtrajs)

        dummy = np.zeros(dtrajs.size)
        data = np.ones(dtrajs.size)
        # using sparse is fast and sucint
        counts = np.ravel(scipy.sparse.coo_matrix((data, (dtrajs, dummy)), shape=(self.numClusters, 1)).toarray())

        clustersToDelete = np.argwhere(counts < clusterCountsThreshold)
        if clustersToDelete.shape[0] > 0:
            print("Removing %d clusters due to a small number of counts (less than %d)" % (clustersToDelete.shape[0], clusterCountsThreshold))
            self.clusterCenters = np.delete(self.clusterCenters, clustersToDelete, axis=0)
            self._writeClusterCenters(self.clusterCenters, self.clusterCentersFile)
            print("Reassigning trajectories")
            self.dtrajs = self.assignNewTrajectories(self.x)

    def _writeClusterCenters(self, clusterCenters, outputFilename):
        np.savetxt(outputFilename, clusterCenters, fmt=b"%.5f")

    def _writeDtrajs(self, filenames, dtrajs, filenameTemplate="%s.disctraj"):
        for filename, dtraj in zip(filenames, dtrajs):
            fname = os.path.split(filename)[-1][:-4]
            dtrajfname = filenameTemplate % (fname)
            np.savetxt(dtrajfname, dtraj, fmt=b"%d")


# Standalone functions

def loadTrajFiles(trajectoryFolder, trajectory_basename):
    trajectoryBasename = os.path.join(trajectoryFolder, trajectory_basename)

    # load traj
    files = glob.glob(trajectoryBasename)
    x = len(files)*[0]
    for i, f in enumerate(files):
        currentX = np.loadtxt(f, ndmin=2)[:, 1:]
        x[i] = currentX
    if not x:
        raise ValueError("Didn't find any trajectory files in the specified path!!!")
    return x, files


def makeFolder(outputDir):
    if not os.path.exists(outputDir):
        os.makedirs(outputDir)



import os
import prody
from string import Template
import shutil
import argparse


def create_symlinks(src, dst):
    if not os.path.islink(dst):
        os.symlink(src, dst)


def installer(schr, pele, pele_exec, pele_license):
    file_input = 'FrAG_PELE/FrAG/constants.py'
    shutil.copy('FrAG_PELE/FrAG/Templates/constants.py', file_input)
    d = {"SCHRODINGER":schr, "PELE":pele, "PELE_BIN":pele_exec, "LICENSE":pele_license }
    filein = open(file_input)
    src = Template( filein.read() )
    installation_content = src.safe_substitute(d)
    filein.close()
    with open(file_input, "w") as f:
        f.write(installation_content)

def retrieve_prody_molecule(pdb, selection_command):
    molecule = prody.parsePDB(pdb)
    molecule_selected = molecule.select(selection_command)
    return molecule_selected

def parse_args():
    parser = argparse.ArgumentParser(description='Installation Script')
    parser.add_argument('--schr', type=str, help='SCRHODINGER main path. i.e /opt/apps/schrodinger-2017/'),
    parser.add_argument('--pele', type=str, help='PELE main path. i.e /opt/apps/PELErev1234/'),
    parser.add_argument('--pele_exec', type=str, help='PELE bin path. i.e /opt/apps/PELErev1234/bin/Pele_mpi'),
    parser.add_argument('--pele_license', type=str, help='PELE licenses PATH. i.e /opt/apps/PELErev12345/licenses/')
    args = parser.parse_args()
    return args.schr, args.pele, args.pele_exec, args.pele_license

if __name__ == "__main__":
    schr, pele, pele_exec, pele_license = parse_args()
    installer(schr, pele, pele_exec, pele_license)



from __future__ import absolute_import, division, print_function, unicode_literals
from AdaptivePELE.simulation import simulationrunner
import AdaptivePELE.adaptiveSampling as adaptiveSampling
import argparse
import os


def automateSimulation(args):
    """
        Run multiple AdaptivePELE simulation with the same parameters, changing
        only the seed

        :param args: Object containing the command line arguments
        :type args: object
    """
    controlFile = args.controlFile
    numSimulations = args.numSimulations
    nProcessors = args.nProcessors
    nSteps = args.nSteps
    simulationName = args.simulationName
    simulationParameters = simulationrunner.SimulationParameters()
    simulationParameters.templetizedControlFile = controlFile
    simulationRunner = simulationrunner.SimulationRunner(simulationParameters)
    epochs = args.epochs
    if epochs:
        rangeOfEpochs = epochs
    else:
        rangeOfEpochs = list(range(1, numSimulations+1))
    print("rangeOfEpochs", rangeOfEpochs)

    for i in rangeOfEpochs:
        controlFileDictionary = {"SEED": "%d%d%d", "OUTPUTPATH": "%s_%d"}
        SEED_i = int(controlFileDictionary["SEED"] % (i, nProcessors, nSteps))
        controlFileDictionary["SEED"] = SEED_i
        outputPath_i = controlFileDictionary["OUTPUTPATH"] % (simulationName, i)
        controlFileDictionary["OUTPUTPATH"] = outputPath_i
        controlFileName = "tmp_%s_controlfile_%s_%d.conf" % (os.path.splitext(controlFile)[0], simulationName, i)
        controlFileName = controlFileName.replace("/", "_")
        simulationRunner.makeWorkingControlFile(controlFileName, controlFileDictionary)
        print("Starting simulation %d" % i)
        adaptiveSampling.main(controlFileName)


def parseArguments():
    """
        Parse the command line arguments
    """
    parser = argparse.ArgumentParser(description="Automate the process "
                                     "of repeating simulations")
    parser.add_argument('controlFile', type=str)
    parser.add_argument('numSimulations', type=int)
    parser.add_argument('nProcessors', type=int)
    parser.add_argument('nSteps', type=int)
    parser.add_argument('simulationName', type=str)
    parser.add_argument("-e", "--epochs", nargs='*', type=int, help="Epochs to run")
    args = parser.parse_args()
    return args


def main():
    """
        Run the multiple simulations
    """
    args = parseArguments()
    automateSimulation(args)

if __name__ == "__main__":
    main()



import sys
import os
import re
import ntpath
import prody
import glob
import argparse
import pandas as pn
import matplotlib.pyplot as plt


def parse_arguments():
    """
            Parse user arguments

            Output: list with all the user arguments
        """
    # All the docstrings are very provisional and some of them are old, they would be changed in further steps!!
    parser = argparse.ArgumentParser(description="""""")
    required_named = parser.add_argument_group('required named arguments')
    # Growing related arguments
    required_named.add_argument("-pdbr", "--pdb_reference",required=True,
                                help="""Reference pdb file.""")
    required_named.add_argument("-p", "--path", required=True,
                                help="""Path to folder to be analyzed. """)

    parser.add_argument("-pdbt", "--pdb_target",
                                help="""Target pdb file. """)

    parser.add_argument("-wp", "--w2pdb", default=False,
                        help="If true export a pdb file with the alignment result.")
    parser.add_argument("-ch", "--chain", default="L",
                        help="Name of the ligand's chain")
    parser.add_argument("-o", "--out", default="report_rmsd.txt",
                        help="Output file name.")
    parser.add_argument("-c", "--column_to_add", default="Binding Energy",
                        help="Column name of the report file that you want to add to the report")
    # Plot arguments
    parser.add_argument("-x", "--xcol", default="RMSD",
                        help="Column name of the report file that you want to use as X axis in the plot.")
    parser.add_argument("-op", "--out_plot", default="plot.png",
                        help="Filename of the output graph.")

    args = parser.parse_args()

    return args.pdb_reference, args.path, args.pdb_target, args.w2pdb, args.chain, args.out, args.column_to_add, \
           args.xcol, args.out_plot


def superimpose_backbones(pdb_target, pdb_reference):
    # Loading PDB files
    target = prody.parsePDB(pdb_target)
    reference = prody.parsePDB(pdb_reference)
    # Selection of the backbone
    target_backbone = target.select("backbone and not water")
    reference_backbone = reference.select("backbone and not water")
    # Obtain common residues
    common_residues = check_common_residues(target_backbone, reference_backbone)
    common_residues = [str(residue) for residue in common_residues]
    # Select only common residues
    target_common = target_backbone.select("resnum {}".format(' '.join(common_residues)))
    reference_common = reference_backbone.select("resnum {}".format(' '.join(common_residues)))
    # Superimpose the target to the reference backbone
    transformation = prody.calcTransformation(target_common, reference_common)
    prody.applyTransformation(transformation, target)
    prody.writePDB("/home/carlespl/project/growing/Ligand_growing/Analysis/check_target.pdb", target)
    prody.writePDB("/home/carlespl/project/growing/Ligand_growing/Analysis/check_ref.pdb", reference)
    return target, reference


def check_common_residues(pdb_target_prody, pdb_reference_prody):
    list_of_types_and_resnums_target= []
    list_of_types_and_resnums_reference = []
    for resnum, resname in zip(pdb_target_prody.getResnums(), pdb_target_prody.getResnames()):
        list_of_types_and_resnums_target.append((resnum, resname))
    for resnum, resname in zip(pdb_reference_prody.getResnums(), pdb_reference_prody.getResnames()):
        list_of_types_and_resnums_reference.append((resnum, resname))
    common_residues = sorted(list(set(list_of_types_and_resnums_target).intersection(list_of_types_and_resnums_reference)))
    only_resnums = []
    for residue in common_residues:
        resnum = residue[0]
        only_resnums.append(resnum)
    return only_resnums


def ligands_rmsd_calculator(pdb_target, pdb_reference, column_to_add=None, write2pdb=False, ligand_chain="L", ):
    """

    :param pdb_target: problem pdb file
    :param pdb_reference: reference pdb file
    :param write2report: if True export results in a file
    :param write2pdb: pdb file with the result of the superposition between "pdb_target" and "pdb_reference"
    :param ligand_chain: name of the chain of the ligand
    :return: superpose the backbone of the pdb_target to the pdb_reference and computes the RMSD of the ligand
    """
    repair_pdbs(pdb_target)
    target, reference = superimpose_backbones(pdb_target, pdb_reference)
    target_ligand = target.select("chain {} and heavy".format(ligand_chain))
    reference_ligand = reference.select("chain {} and heavy".format(ligand_chain))

    # Creating temporary files to perform RMSD computations in futher steps
    prody.writePDB("structure_superposed_target_tmp.pdb", target_ligand)
    prody.writePDB("structure_superposed_reference_tmp.pdb", reference_ligand)
    sort_atoms_by_name("structure_superposed_reference_tmp.pdb")
    results = []
    for i, pdb in enumerate(check_if_multimodels_and_split("structure_superposed_target_tmp.pdb")):
        with open("structure_superposed_target_tmp.pdb", "w") as write_pdb:
            write_pdb.write(pdb)
        sort_atoms_by_name("structure_superposed_target_tmp.pdb")
        target_sorted = prody.parsePDB("structure_superposed_target_tmp.pdb")
        reference_sorted = prody.parsePDB("structure_superposed_reference_tmp.pdb")
        RMSD = prody.calcRMSD(reference_sorted, target_sorted)
        print(RMSD)
        if column_to_add:
            column = get_report_column_from_trajectory(pdb_target, "report_", column_to_add, i)
            result_sumary = "{}\t{}\t{:2.3f}\t{}".format(pdb_target, i, RMSD, column)
        else:
            result_sumary = "{}\t{}\t{:2.3f}\t ".format(pdb_target, i, RMSD)
        results.append(result_sumary)

    if write2pdb:
        prody.writePDB("structure_superposed.pdb", target+reference)
    # Deleting temporary files
    os.remove("structure_superposed_target_tmp.pdb")
    os.remove("structure_superposed_reference_tmp.pdb")
    return results


def repair_pdbs(pdb_file):
    with open(pdb_file) as pdb:
        pdb_content = pdb.readlines()
    pdb_new = []
    for line in pdb_content:
        atom_name = line[12:16]
        if line.startswith("HETATM") and atom_name.strip().startswith("G"):
            line_new = list(line)
            size = len(atom_name.strip())
            white_spaces = 3-size
            line_new[12:16] = line[77:78] + atom_name.strip() + " "*white_spaces
            modified = "".join(line_new)
            pdb_new.append(modified)
        else:
            pdb_new.append(line)
    pdb_modified = "".join(pdb_new)
    with open(pdb_file, "w") as write_output:
        write_output.write(pdb_modified)


def sort_atoms_by_name(pdb_file):
    with open(pdb_file) as pdb:
        pdb_content = pdb.readlines()
    pdb_content_sorted = (sorted(pdb_content, key=lambda atom_name: atom_name[12:16]))
    new_pdb_content = "".join(pdb_content_sorted)
    with open(pdb_file, "w") as write_output:
        write_output.write(new_pdb_content)


def compute_rmsd_in_serie(pdb_reference, path, column_to_add, ligand_chain="L", output_file="report_rmsd.txt"):
    equilibration_files = glob.glob("{}/equilibration*/trajectory*.pdb".format(path))
    if os.path.exists(os.path.join(path, output_file)):
        os.remove(os.path.join(path, output_file))
    for file in equilibration_files:
        result = ligands_rmsd_calculator(file, pdb_reference, column_to_add, write2pdb=False, ligand_chain=ligand_chain)
        for line in result:
            if os.path.exists(os.path.join(path, output_file)):
                with open(os.path.join(path, output_file), "a") as writing_file:
                    writing_file.write("{}\n".format(line))
            else:
                with open(os.path.join(path, output_file), "w") as writing_file:
                    writing_file.write("{}\n".format(line))


def check_if_multimodels_and_split(pdb_file):
    with open(pdb_file) as pdb:
        content = pdb.readlines()
    pdbs = []
    lines_in_model = []
    for line in content:
        lines_in_model.append(line)
        if "ENDMDL" in line:
            pdbs.append("".join(lines_in_model))
            lines_in_model = []
    return pdbs


def get_report_file_from_trajectory(trajectory_path, report_prefix):
    regex = re.compile(r'\d+')
    sufix = regex.findall(trajectory_path)[-1]
    prefix = trajectory_path.split("/")[0:-1]
    new_prefix = "/".join(prefix)
    path_completed = os.path.join(new_prefix, report_prefix+sufix)
    return path_completed


def get_report_column_from_trajectory(trajectory_path, report_prefix, column, model):
    report_path = get_report_file_from_trajectory(trajectory_path, report_prefix)
    report_data = pn.read_csv(report_path, sep="    ")
    return report_data[column][model]


def plot_results(report_file, x, y, xcolumn, ycolumn, output_name):
    data = pn.read_csv(report_file, sep="\t")
    data.plot(x, y, kind="scatter")
    plt.xlabel(xcolumn)
    plt.ylabel(ycolumn)
    print(output_name)
    plt.savefig(output_name)
    plt.show()


def get_snapshot(trajectory_file, model, output_file):
    pdb = check_if_multimodels_and_split(trajectory_file)[model]
    with open(output_file,"w") as pdb2write:
        pdb2write.write(pdb)


def get_min_row(data, column_number):
    minimum = data.loc[data[column_number] == data[column_number].min()]
    return minimum


def get_pdb_by_min_value(report_file, column_number):
    data = pn.read_csv(report_file, sep="\t", header=None)
    minium_row = get_min_row(data, column_number)
    return minium_row[0].values[0], minium_row[1].values[0]


if __name__ == '__main__':
    pdb_reference, path, pdb_target, w2pdb, chain, output_file, column_to_add, xcol, out_plot = parse_arguments()
    compute_rmsd_in_serie(pdb_reference, path, column_to_add, ligand_chain="L", output_file=output_file)
    get_snapshot(get_pdb_by_min_value(os.path.join(path, output_file), 2)[0],
                 get_pdb_by_min_value(os.path.join(path, output_file), 2)[1],
                 os.path.join(path, "minimum_rmsd.pdb"))
    get_snapshot(get_pdb_by_min_value(os.path.join(path, output_file), 3)[0],
                 get_pdb_by_min_value(os.path.join(path, output_file), 3)[1],
                 os.path.join(path, "minimum_{}.pdb".format(column_to_add.replace(" ", ""))))
    plot_results(os.path.join(path, output_file), 2, 3, xcol, column_to_add, os.path.join(path, out_plot))




import subprocess
import numpy
import sys
import shutil
# subprocess.call("pip install numpy cython".split())
from setuptools import setup, find_packages, Command
from string import Template
# To use a consistent encoding
from codecs import open
from os import path
from distutils.extension import Extension
from setuptools.command.install import install
try:
    from Cython.Build import cythonize
    from Cython.Distutils import build_ext
except ImportError:
    use_cython = False
else:
    use_cython = True
from distutils.command.sdist import sdist as _sdist

# Run the following line to compile atomset package
# python setup.py build_ext --inplace
import frag_pele.constants as constants


class PreInstallCommand(install):
    description = "Installer"
    user_options = install.user_options + [
        ('schr=', None, 'SCRHODINGER main path. i.e /opt/apps/schrodinger-2017/'),
        ('pele=', None, 'PELE main path. i.e /opt/apps/PELErev1234/'),
        ('pele-exec=', None, 'PELE bin path. i.e /opt/apps/PELErev1234/bin/Pele_mpi'),
        ('pele-license=', None, 'PELE licenses PATH. i.e /opt/apps/PELErev12345/licenses/')
    ]

    def initialize_options(self):
        install.initialize_options(self)
        self.schr = None
        self.pele = None
        self.pele_exec = None
        self.pele_license = None

    def finalize_options(self):
        install.finalize_options(self)
        #if not self.schr:
        #    raise ValueError("Define --schr path. Check --help-commands for more help")
        #if not self.pele:
        #    raise ValueError("Define --pele path. Check --help-commands for more help")
        #if not self.pele_exec:
        #    raise ValueError("Define --pele-exec path. Check --help-commands for more help")
        #if not self.pele_license:
        #    raise ValueError("Define --pele-license path. Check --help-commands for more help")
        #if not self.mpirun:
        #    raise ValueError("Define --mpirun path. Check --help-commands for more help")

    def run(self):
        #print("Cythonazing")
        #subprocess.call("python frag_pele/setup.py build_ext --inplace".split())
        #print("Installing packages")
        #subprocess.call("pip install {}".format(" ".join(packages)).split())
        print("Setting environmental variables")
        installer(self.schr, self.pele, self.pele_exec, self.pele_license)
        print("Install")
        install.run(self)

class PostInstallCommand(install):
    """Post-installation for installation mode."""
    def run(self):
        install.run(self)


def installer(schr, pele, pele_exec, pele_license):
    file_input = 'frag_pele_pele/constants.py'
    shutil.copy('frag_pele_pele/Templates/constants.py', file_input)
    d = {"SCHRODINGER":schr, "PELE":pele, "PELE_BIN":pele_exec, "LICENSE":pele_license }
    filein = open(file_input)
    src = Template( filein.read() )
    installation_content = src.safe_substitute(d)
    filein.close()
    with open(file_input, "w") as f:
        f.write(installation_content)

        

here = path.abspath(path.dirname(__file__))
ext_modules = []
cmdclass = {}
#cmdclass.update({'install': PreInstallCommand})


class sdist(_sdist):
    def run(self):
        # Make sure the compiled Cython files in the distribution are
        # up-to-date
        from Cython.Build import cythonize
        cythonize(['cython/mycythonmodule.pyx'])
        _sdist.run(self)
        cmdclass['sdist'] = sdist

        

if use_cython:
    ext_modules += [
        Extension("frag_pele.AdaptivePELE_repo.AdaptivePELE.atomset.atomset", ["frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset/atomset.pyx"], include_dirs=["frag_pele/AdaptivePELE_repo/AdaptivePELE", "frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset"]),
        Extension("frag_pele.AdaptivePELE_repo.AdaptivePELE.atomset.SymmetryContactMapEvaluator", ["frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset/SymmetryContactMapEvaluator.pyx"], include_dirs=["frag_pele/AdaptivePELE_repo/AdaptivePELE", "frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset"]),
        Extension("frag_pele.AdaptivePELE_repo.AdaptivePELE.atomset.RMSDCalculator", ["frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset/RMSDCalculator.pyx"], include_dirs=["frag_pele/AdaptivePELE_repo/AdaptivePELE", "frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset"]),
        Extension("frag_pele.AdaptivePELE_repo.AdaptivePELE.freeEnergies.utils", ["frag_pele/AdaptivePELE_repo/AdaptivePELE/freeEnergies/utils.pyx"], include_dirs=["frag_pele/AdaptivePELE_repo/AdaptivePELE", "frag_pele/AdaptivePELE_repo/AdaptivePELE/freeEnergies"])
    ]
    cmdclass.update({'build_ext': build_ext})
else:
    ext_modules += [
        Extension("frag_pele.AdaptivePELE_repo.AdaptivePELE.atomset.atomset", ["frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset/atomset.c"], include_dirs=["frag_pele/AdaptivePELE_repo/AdaptivePELE", "frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset"]),
        Extension("frag_pele.AdaptivePELE_repo.AdaptivePELE.atomset.SymmetryContactMapEvaluator", ["frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset/SymmetryContactMapEvaluator.c"], include_dirs=["frag_pele/AdaptivePELE_repo/AdaptivePELE", "frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset"]),
        Extension("frag_pele.AdaptivePELE_repo.AdaptivePELE.atomset.RMSDCalculator", ["frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset/RMSDCalculator.c"], include_dirs=["frag_pele/AdaptivePELE_repo/AdaptivePELE", "frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset"]),
        Extension("frag_pele.AdaptivePELE_repo.AdaptivePELE.freeEnergies.utils", ["frag_pele/AdaptivePELE_repo/AdaptivePELE/freeEnergies/utils.c"], include_dirs=["frag_pele/AdaptivePELE_repo/AdaptivePELE", "frag_pele/AdaptivePELE_repo/AdaptivePELE/freeEnergies"])
    ]

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()
setup(
    name="frag_pele",
    version="1.1.0.3",
    description='FrAG, a new tool for in silico hit-to-lead drug design, capable of growing a frag_pelement into a core while exploring the protein-ligand conformational space',
    long_description=long_description,
    url="https://github.com/carlesperez94/frag_pele/",
    author='Carles Perez Lopez, Daniel Soler Viladrich',
    author_email='daniel.soler@nostrumbiodiscovery.com, carlesperez@gmail.com',
    license='',
    packages=find_packages(exclude=['docs', 'tests']),
    package_data={"frag_pele/AdaptivePELE_repo/AdaptivePELE/atomset": ['*.pxd'], "Templates": ["*.pdb", "*.conf"] },
    include_package_data=True,
    include_dirs=[numpy.get_include()],
    install_requires=['cython', 'numpy',  'scipy', 'matplotlib', 'biopython ', 'pandas',  'prody', 'six'],
    cmdclass=cmdclass,
    ext_modules=ext_modules,  # accepts a glob pattern
    #include_dirs=[numpy.get_include()],
    classifiers=(
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Intended Audience :: Science/Research"
    ),
    project_urls={
    'Documentation': 'https://carlesperez94.github.io/frag_pele/',
    'Source': 'https://carlesperez94.github.io/frag_pele/',
'Tracker': 'https://github.com/carlesperez94/frag_pele/issues',
},
)




from __future__ import absolute_import, division, print_function, unicode_literals
from AdaptivePELE.constants import blockNames
from abc import abstractmethod
from AdaptivePELE.spawning import densitycalculatortypes
import sys


def continousDensity(contacts):
    if contacts > 1.0:
        return 8.0
    else:
        return 64.0/(-4*contacts+6)**3


class DensityCalculatorBuilder():
    def build(self, spawningBlock):
        """
            Build the DensityCalculator object according to the parameters selcted

            :param spawningBlock: Spawning block of the control file
            :type spawningBlock: dict

            :returns: :py:class:`.DensityCalculator` -- DensityCalculator object selected
        """
        try:
            densityBlock = spawningBlock[blockNames.SpawningParams.density]
        except KeyError:
            print("Using null density calculator (no preference for any cluster)")
            return NullDensityCalculator()

        try:
            density_type = densityBlock[blockNames.DensityCalculator.type]
        except KeyError:
            sys.exit("Density calculator must have a type")

        if density_type == blockNames.DensityCalculator.null or density_type == blockNames.DensityCalculator.constant:
            print("Using constant density")
            return NullDensityCalculator()
        elif density_type == blockNames.DensityCalculator.heaviside:
            try:
                paramsBlock = densityBlock[blockNames.DensityCalculator.params]
                values = paramsBlock[blockNames.DensityCalculatorParams.values]
                conditions = paramsBlock[blockNames.DensityCalculatorParams.conditions]
                return DensityCalculatorHeaviside(conditions, values)
            except KeyError:
                print("Using default parameters for Heaviside density calculator")
                return DensityCalculatorHeaviside()
        elif density_type == blockNames.DensityCalculator.continuous:
            print("Using continuous density calculator")
            return ContinuousDensityCalculator()
        elif density_type == blockNames.DensityCalculator.exitContinuous:
            print("Using inverse continuous density calculator")
            return ExitContinousDensityCalculator()
        else:
            sys.exit("Unknown density calculator type! Choices are: " + str(densitycalculatortypes.DENSITY_CALCULATOR_TYPE_TO_STRING_DICTIONARY.values()))


class DensityCalculator():
    def __init__(self):
        self.type = "BaseClass"

    @abstractmethod
    def calculate(self, contacts, contactThreshold):
        pass


class DensityCalculatorHeaviside(DensityCalculator):
    # Mostly duplicated code with threshold calculator
    def __init__(self, conditions=None, values=None):
        DensityCalculator.__init__(self)
        self.type = densitycalculatortypes.DENSITY_CALCULATOR_TYPES.heaviside
        if conditions is None:
            conditions = []
        if values is None:
            values = [1.]

        if len(values) != len(conditions) and len(values) != len(conditions) + 1:
            raise ValueError('The number of values must be equal or one more, than the number of conditions')

        self.conditions = conditions
        self.values = values

    def calculate(self, contacts, contactThreshold):
        """
            Calcuate the density value according to the contact ratio

            :param contacts: Contacts ratio
            :type contacts: float
            :param contactThreshold: Deprecated parameter
            :type contactThreshold: float

            :returns: float -- Density value for the value of the contacts ratio
        """
        for i in range(len(self.conditions)):
            # change, so that whole condition is in array
            if contacts > self.conditions[i]:
                # ensure that the value is a float
                return float(self.values[i])
        # the way it's built, it makes more sense to return this value, but,
        # should check that len(value) = len(conditions) + 1 in order to
        # return the "else" value
        return self.values[-1]


class NullDensityCalculator(DensityCalculator):
    def __init__(self):
        DensityCalculator.__init__(self)
        self.type = densitycalculatortypes.DENSITY_CALCULATOR_TYPES.null

    def calculate(self, contacts, contactThreshold):
        """
            Calcuate the density value according to the contact ratio, in this
            case is always one

            :param contacts: Contacts ratio
            :type contacts: float
            :param contactThreshold: Deprecated parameter
            :type contactThreshold: float

            :returns: float -- Density value for the value of the contacts ratio
        """
        return 1.


class ContinuousDensityCalculator(DensityCalculator):
    def __init__(self):
        DensityCalculator.__init__(self)
        self.type = densitycalculatortypes.DENSITY_CALCULATOR_TYPES.continuous

    def calculate(self, contacts, contactThreshold):
        """
            Calcuate the density value according to the contact ratio

            :param contacts: Contacts ratio
            :type contacts: float
            :param contactThreshold: Deprecated parameter
            :type contactThreshold: float

            :returns: float -- Density value for the value of the contacts ratio
        """
        return continousDensity(contacts)


class ExitContinousDensityCalculator(DensityCalculator):
    def __init__(self):
        DensityCalculator.__init__(self)
        self.type = densitycalculatortypes.DENSITY_CALCULATOR_TYPES.exitContinous

    def calculate(self, contacts, contactThreshold):
        # alternative possibility
        # if contacts > 1.0:
        #     return 1/4.0
        # elif contacts < 0.24:
        #     return 4.0
        # else:
        #     return (-4*contacts+6)**3/32.0

        return 1.0/continousDensity(contacts)



from __future__ import absolute_import, division, print_function, unicode_literals
from builtins import range
import os
import numpy as np
import shutil
import matplotlib.pyplot as plt
from AdaptivePELE.freeEnergies import estimateDG


def lengthVsNtrajs(nruns, lagtime, clusters, lengths, ntrajs, outputFilename, cache, skipFirstSnaphots):
    nLengths = len(lengths)
    nNtrajs = len(ntrajs)
    results = np.zeros((nLengths, nNtrajs))
    stdDev = np.zeros((nLengths, nNtrajs))
    db = np.zeros((nLengths, nNtrajs))
    stdDb = np.zeros((nLengths, nNtrajs))
    for i, length in enumerate(lengths):
        for j, ntraj in enumerate(ntrajs):
            if (length, ntraj) in cache:
                print("Loading cached computation for length:%d and ntrajs:%d" % (length, ntraj))
                results[i][j], stdDev[i][j], db[i][j], stdDb[i][j] = cache[(length, ntraj)]
                with open(outputFilename, 'a') as f:
                    f.write("%d %d %f %f %f %f\n" % (length, ntraj, results[i][j], stdDev[i][j], db[i][j], stdDb[i][j]))
                continue
            print("Computing for length:%d and ntrajs:%d" % (length, ntraj))
            parameters = estimateDG.Parameters(ntrajs=ntraj, length=length, lagtime=lagtime, nclusters=clusters, nruns=nruns,
                                               useAllTrajInFirstRun=False, computeDetailedBalance=True, trajWildcard="traj_*",
                                               folderWithTraj="rawData", skipFirstSteps=skipFirstSnaphots)

            origFilesWildcard = os.path.join(parameters.folderWithTraj, parameters.trajWildcard)
            estimateDG.copyWorkingTrajectories(origFilesWildcard, length, ntraj, bootstrap=True)
            results[i][j], stdDev[i][j], db[i][j], stdDb[i][j] = estimateDG.estimateDG(parameters)
            with open(outputFilename, 'a') as f:
                f.write("%d %d %f %f %f %f\n" % (length, ntraj, results[i][j], stdDev[i][j], db[i][j], stdDb[i][j]))
    return results, stdDev, db, stdDb


def saveResultsFileBckp(outputFilename):
    i = 1
    bckpFilename = outputFilename+".%d.bckp"
    while os.path.isfile(bckpFilename % i):
        i += 1
    try:
        shutil.copy(outputFilename, bckpFilename % i)
    except IOError:
        pass


def plotIsocostLines(extent, minCost, maxCost, steps=10):
    # d = (maxCost - minCost) / steps
    # costs = np.arange(minCost, maxCost, d)
    minCost = np.log10(minCost)
    maxCost = np.log10(maxCost)
    costs = np.logspace(minCost, maxCost, num=steps)
    for cost in costs:
        x = np.arange(extent[0], extent[1], 1)
        y = cost / x
        plt.plot(x, y, color="black")


def main():
    plt.style.use('ggplot')

    lagtime = 100
    clusters = 100

    ilengths = 200
    flengths = 4000
    dlengths = 200
    lengths = list(range(ilengths, flengths, dlengths))
    itrajs = 32-1
    ftrajs = 1008
    dtrajs = 32
    ntrajs = list(range(itrajs, ftrajs, dtrajs))
    nruns = 10
    skipFirstSnaphots = 0
    outputFilename = "results.txt"
    cache = {}
    if os.path.exists(outputFilename):
        with open(outputFilename) as fr:
            for line in fr:
                contents = line.rstrip().split()
                if not contents[0].isdigit():
                    continue
                l, traj, dg, stdDG, db, stdDB = map(float, contents)
                cache[(int(l), int(traj))] = (dg, stdDG, db, stdDB)

    saveResultsFileBckp(outputFilename)

    with open(outputFilename, 'w') as f:
        f.write("Computing DG, stdDG, DB and stdDB for different lengths and number of trajectories\n")
        f.write("Lengths: %s\n" % lengths)
        f.write("Ntrajs: %s\n" % ntrajs)
        f.write("Skipping first: %d snapshots of each trajectory\n" % skipFirstSnaphots)
    results, stdDev, db, stdDb = lengthVsNtrajs(nruns, lagtime, clusters, lengths, ntrajs, outputFilename, cache, skipFirstSnaphots)
    np.save("results.npy", results)
    np.save("stdDev.npy", stdDev)
    np.save("db.npy", db)
    np.save("stdDb.npy", stdDb)

    # results = np.load("results.npy")

    extent = [itrajs+1-dtrajs/2, ftrajs+dtrajs/2, ilengths-dlengths/2, flengths-dlengths/2]  # +1 for aesthetical purposes
    plt.figure(1)
    plotIsocostLines(extent, (ilengths+dlengths)*(itrajs+dtrajs), (flengths-dlengths)*(ftrajs-dtrajs), 6)
    plt.imshow(results, interpolation="nearest", origin="lower", aspect="auto", extent=extent)
    # plt.imshow(results, interpolation="nearest", origin="lower", aspect="auto", extent=extent, vmin=-7, vmax=-5)
    plt.colorbar()
    plt.savefig("dgGrid_1008_4000_rough.png")
    plt.figure(2)
    plotIsocostLines(extent, (ilengths+dlengths)*(itrajs+dtrajs), (flengths-dlengths)*(ftrajs-dtrajs), 6)
    plt.imshow(results, interpolation="bilinear", origin="lower", aspect="auto", extent=extent)
    plt.colorbar()
    plt.savefig("dgGrid_1008_4000_finer.png")
    plt.show()


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function, unicode_literals
from AdaptivePELE.clustering import thresholdcalculator
import unittest
import numpy as np


class thresholdCalculatorTest(unittest.TestCase):
    def testConstantDefaultParams(self):
        thresholdCalculatorBuilder = thresholdcalculator.ThresholdCalculatorBuilder()

        clusteringBlock = {
            "type": "irrelevant",
            "thresholdCalculator": {
                "type": "constant"
            }
        }

        thresholdCalculator = thresholdCalculatorBuilder.build(clusteringBlock)
        value = thresholdCalculator.value

        goldenValue = 2.
        self.assertAlmostEqual(value, goldenValue, 10)

    def testConstantOtherParams(self):
        thresholdCalculatorBuilder = thresholdcalculator.ThresholdCalculatorBuilder()

        clusteringBlock = {
            "type": "irrelevant",
            "thresholdCalculator": {
                "type": "constant",
                "params": {
                    "value": 3
                }
            }
        }

        thresholdCalculator = thresholdCalculatorBuilder.build(clusteringBlock)
        value = thresholdCalculator.value

        goldenValue = 3.
        self.assertAlmostEqual(value, goldenValue, 10)

    def testDefaultThresholdCalculator(self):
        thresholdCalculatorBuilder = thresholdcalculator.ThresholdCalculatorBuilder()

        clusteringBlock = {
            "type": "irrelevant"
        }

        thresholdCalculator = thresholdCalculatorBuilder.build(clusteringBlock)
        values = thresholdCalculator.values
        conditions = thresholdCalculator.conditions

        goldenValues = [2., 3., 4, 5]
        goldenConditions = [1.0, 0.75, 0.5]
        np.testing.assert_almost_equal(np.array(values), np.array(goldenValues), 10)
        np.testing.assert_almost_equal(np.array(conditions), np.array(goldenConditions), 10)

    def testHeavisideDefaultParams(self):
        thresholdCalculatorBuilder = thresholdcalculator.ThresholdCalculatorBuilder()

        clusteringBlock = {
            "type": "irrelevant",
            "thresholdCalculator": {
                "type": "heaviside"
            }
        }

        thresholdCalculator = thresholdCalculatorBuilder.build(clusteringBlock)
        values = thresholdCalculator.values
        conditions = thresholdCalculator.conditions

        goldenValues = [2., 3, 4., 5.]
        goldenConditions = [1.0, 0.75, 0.5]
        np.testing.assert_almost_equal(np.array(values), np.array(goldenValues), 10)
        np.testing.assert_almost_equal(np.array(conditions), np.array(goldenConditions), 10)

    def testHeavisideParams(self):
        thresholdCalculatorBuilder = thresholdcalculator.ThresholdCalculatorBuilder()

        clusteringBlock = {
            "type": "irrelevant",
            "thresholdCalculator": {
                "type": "heaviside",
                "params": {
                    "values": [2, 5, 10],
                    "conditions": [3, 10]
                }
            }
        }

        thresholdCalculator = thresholdCalculatorBuilder.build(clusteringBlock)
        values = thresholdCalculator.values
        conditions = thresholdCalculator.conditions

        goldenValues = [2., 5., 10.]
        goldenConditions = [3., 10.]
        self.assertAlmostEqual(values, goldenValues, 10)
        self.assertAlmostEqual(conditions, goldenConditions, 10)
